package ExSite::Dialog;

use strict;
use ExSite::Config;
use Class::Struct;
use ExSite::Form;

struct("ExSite::Dialog"=>{
	default_image_dimension=>'$',
	db=>'ExSite::Form',
});
# Default max image dimension
sub default_image_dimension{
	my ($this,$value) = @_;
	# Default image dimension
	my $default = 200;
	if (!$value){
		# Return default image dimension
		return $this->{default_dim} if $this->{default_dim};
		# Set default image dimension if there isn't one exists
		$this->{default_dim} = $default;
		return $this->{default_dim};
	}
	# Set new image dimension
	$this->{default_dim} = $value;
	return $this->{default_dim};
}
# Fetch pages
sub fetch_page{
	my ($this,$query) = @_;
	my $sectionid;
	if ($query->{section_id}){
		# Found the section id from the query string
		$sectionid = $query->{section_id};
	}else{
		if (exists $query->{data_id}){
			# Figure out the section id by using the content data object
			require ExSite::ContentData;
			my $cd = new ExSite::ContentData(id=>$query->{data_id});
			$cd->set_context();
			$sectionid = $cd->id("section");
		}else{
			# Figure out the section id by using the Page object
			require ExSite::Page;	
			my $cd = new ExSite::Page(id=>$query->{cont_id});
			$cd = $cd->get();
			$sectionid = $cd->{section_id};
		}
	}
	# Fetch pages
	my @pages = $this->db->fetch_match("page",{section_id=>$sectionid,type=>"page"},'title');
	return \@pages;
}
# Figure out section id
sub find_section_id{
	my ($this,$query) = @_;
	my $cdata;
	if (exists($query->{data_id})){
		require ExSite::ContentData;
		$cdata = new ExSite::ContentData(id=>$query->{data_id});
	}else{
		require ExSite::Page;
		$cdata = new ExSite::Page(id=>$query->{cont_id});
	}
	$cdata->set_context();
	return $cdata->get("section","section_id");
}
# Adjust the size of the image
sub adjust_image_size{
	# $image_tag: html image tag
	# $fdata: encoded image data
	my ($this,$image_tag,$fdata) = @_;
	require ExSite::Image;
	my $defaultimagedim = $this->default_image_dimension();
	my $image = new ExSite::Image($fdata);
	my ($width,$height) = $image->dim();
	# Don't resize the image if its length and width are less or equal to 200
	return $image_tag if (($width <= $defaultimagedim) && ($height <= $defaultimagedim));
	if ($width >= $height){
		# Set width to 200 and adjust the height using the width ratio(width / default image dimension($defaultimagedim))
		$height = $this->shrink_image_dimension($width,$height);
		$width = $defaultimagedim;
	}else{
		# Set height to 200 and adjust the width using the height ratio(height / default image dimension($defaultimagedim))
		$width = $this->shrink_image_dimension($height,$width);
		$height = $defaultimagedim;
	}
	my $new_tag = $image_tag;	
	# Remove existing height attribute
	$new_tag =~ s/height=.+?(\s|>)//gi;
	# Remove existing width attribute
	$new_tag =~ s/width=.+?(\s|>)//gi;
	# Remove the '>' in the html image tag
	$new_tag =~ s/>$//g;
	$new_tag .= " height=\"$height\" width=\"$width\">";
	return $new_tag;
}
# Shrink image dimension
sub shrink_image_dimension{ 
	my ($this,$reference_size,$need_adjustment) = @_;
	# Default image dimension
	my $defaultimagedim = $this->default_image_dimension();
	# Height/width or width/height ratio
	my $reduce_ratio = $reference_size / $defaultimagedim;
	my $final_size = $need_adjustment / $reduce_ratio;
	return int($final_size);
}
# Get object filename
sub get_object_filename{
	# $id: content id
	my ($this,$id) = @_;
	return if !$id;
	require ExSite::Content;
	my $c = new ExSite::Content(id=>$id);
	return $c->filename();
}
# Get library object
sub get_object{
	# $type: object type(either an image or an url)
	# $id: content id
	my ($this,$type,$id) = @_;
	return if !$id;
	require ExSite::Content;
	# Create a content object
	my $c = new ExSite::Content(id=>$id);
	$c->set_context("child");	
	my $link .= $c->get_html;
	my $out = "content-type: text/html\n\n";
	if ($type eq "image"){
		my $fdata = $c->get("content_data");
		# Adjust image dimension
		$out .= $this->adjust_image_size($link,$fdata->{fdata});
	}elsif ($type eq "url"){
		if ($link =~ /img/i){
			$out .= $c->get_url;
		}else{
			$out .= "$config{server}->{HTMLpath}/_ExSite/images/checker.png";
		}
	}
	return $out;
}
# Fetch libraries
sub fetch_library{
	# $sectionid: section id
	my ($this,$sectionid) = @_;
	return [] if !$sectionid;
	require ExSite::Section;
	my $s = new ExSite::Section(id=>$sectionid);
	# fetch libraries
	my @records = $s->fetch_libraries;
	return \@records;
}
# Is this a file in the library?
sub is_exist{
	# $sectionid: section id
	# $name: name of the object
	my ($this,$sectionid,$name) = @_;
	return 0 if !$sectionid || !$name;
	# Fetch section information
	require ExSite::Section;
	# Create a section object
	my $section = new ExSite::Section(id=>$sectionid);
	# Set context
	$section->set_context();
	# Search for the file
	my $c = $section->find($name);
	return 0 if !$c;
	return 1 if $c->id();
	return 0;
}
# Truncate text
sub truncate_text{
	# $text: text
	# $num: character limit
	my ($this,$text,$num) = @_;
	my $default = 150;
	my $cutoff = $num || $default;
	# Truncate the text
	if (length($text) > $cutoff){
		$text = substr($text,0,$cutoff);
		$text .= "...";
	}
	return $text;
}
1;
