#------------------------------------------------------------------------
# ExSite::Excel
#
# Excel integration tools
#
# Methods:
#   ExcelExport : takes an array of datahashes, and exports them to
#                 an MS-Excel file which can be written to disk or stdout
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package ExSite::Excel;
use strict;
use Carp;

require Exporter;
use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(&ExcelExport);

#------------------------------------------------------------------------
# Excel tools - for importing/exporting Excel spreadsheets
# 
# $name is the excel file you want to write to, or "-" for stdout
# $records is the reference to the record you want to write to excel
# $col_to_print is the columns in the recored you want to print to excel; 
#    if not defined, writes all the columns.
# say we have @col_to_print=(A;B), where A is the name of the 
# corresponding column you want it printed on excel and B is the actual 
# name of the column in the database
#------------------------------------------------------------------------

sub ExcelExport{
    my ($records, $name, $col_to_print,$title)=@_;

    require Spreadsheet::WriteExcel;
    #require Spreadsheet::ParseExcel;
    #require Spreadsheet::ParseExcel::Simple;

    my $workbook=Spreadsheet::WriteExcel->new($name);    
    my $worksheet=$workbook->addworksheet();

    my $format=$workbook->addformat();
    my $increment;
    if (defined $title){$increment=1; $worksheet->write(0,0,$title,$format);}
    else{$increment=0;}
    my $i=0+$increment;
    my $j;
    my $passed=0;
    
    my @r_record;
    $format->set_bold();
    
    if (ref($records) eq "HASH"){@r_record=keys(%$records);}
    elsif (ref($records) eq "ARRAY"){@r_record=@$records;}
    else{ croak "record does not match the required format in Excel::Export<br>";}
    foreach my $temp(@r_record) #used to be @$records
    {
	$j=0;
	if (defined $col_to_print->[0]) 
	{
	    foreach my $temp2(@$col_to_print)
	    {
		my @splitlist=split(/;/, $temp2);
		if ($passed==0)
		{
		    $worksheet->write(0+$increment,$j,$splitlist[0],$format);
 		}
		if (ref($records) eq "HASH"){
		    $worksheet->write($i+1,$j,$records->{$temp}->{$splitlist[1]});
		}
		elsif (ref($records) eq "ARRAY"){
		    $worksheet->write($i+1,$j,$temp->{$splitlist[1]});
		}
		$j++;
	    }
	}
	else
	{
	    foreach my $temp2(keys(%{$temp}))
	    {
		if ($passed==0){
		    $worksheet->write(0+$increment,$j,$temp2); #prints the titles of cols
		}
		if (ref($records) eq "ARRAY"){
		    $worksheet->write($i+1,$j,$temp->{$temp2}); #prints the contents if the record is an ref. to an array
		}
		elsif (ref($records) eq "HASH"){
		    $worksheet->write($i+1,$j,$records->{$temp}->{$temp2}); #prints the contents if the record is an ref. to a hash
		}
		$j++;
	    }
	}
	$passed=1;
	$i++;
    }
    $workbook->close();
}

1;
