package Modules::ExLib;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

=pod

ExLib : a library of general-purpose dynamic content tools

=cut

use strict;
use ExSite::Config;

# inherit from Base class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub read {
    # do nothing
    return;
}

# write : accepts a string argument, which is one of the following:
#
# date
# datetime
# year
# server
# server_software
# address
# uptime
# url
# webmaster

sub write {
    my $this = shift;
    $_ = shift;
    if (/^datetime$/) {
	return scalar localtime;
    }
    elsif (/^date$/) {
	my (undef,undef,undef,$mday,$mon,$year,undef,undef,undef) = localtime;
	return ("Jan","Feb","Mar","Apr","Jun","Jul","Aug","Sep","Oct","Nov","Dec")[$mon]." $mday, ".($year + 1900);
    }
    elsif (/^time$/) {
	my (undef,$min,$hour,undef,undef,undef,undef,undef,undef) = localtime;
	return "$hour:$min";
    }
    elsif (/^year$/) {
	return (localtime)[5] + 1900;
    }
    elsif (/^server$/) {
	return $ENV{SERVER_SIGNATURE};
    }
    elsif (/^server_software$/) {
	return $ENV{SERVER_SOFTWARE};
    }
    elsif (/^address$/) {
	return $ENV{SERVER_ADDR};
    }
    elsif (/^uptime$/) {
	return `uptime`;
    }
    elsif (/^url$/) {
	return "$config{server}{server}$config{server}{HTMLpath}";
    }
    elsif (/^webmaster$/) {
	my ($user,$domain) = split /\@/,$config{site}{administrator};
	return "\n<SCRIPT LANGUAGE=\"JavaScript\">\n<!-- //\nuser = \"$user\";\ndomain = \"$domain\";\naddr = user + '\@' + domain;\ndocument.write('<a href=\"mailto:' + addr + '\">' + addr + '</a>');\n// -->\n</SCRIPT>\n";
    }
    # default to date and time
    return scalar localtime;
}

sub ioctl{
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "Miscellaneous dynamic functions.";
    }
    elsif (/DynContentParam/) {
	return "<b>Parameter:</b>
<select name=\"param\">
<option>date</option>
<option>datetime</option>
<option>year</option>
<option>server</option>
<option>servr_software</option>
<option>address</option>
<option>uptime</option>
<option>url</option>
<option>webmaster</option>
</select>\n";
    }
}

1;

