package Modules::Gallery;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# Gallery : tool to browse an image library
#
# Options:
#
# name=ABC     name of library
# width=N      number of images across in the thumb grid
# border=1|0   whether to put a border around the photos
# title=1|0    whether to title the photo album
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Input;
use ExSite::Misc;
use ExSite::Content;
use Modules::BaseDCD;

# inherit basic behaviours from BaseDCD

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# provide our own write() method

sub write {
    my ($this,$options) = @_;
    my $in = new ExSite::Input;
    my %opt = &DecodeString($options);
    my $db = $share{DB};
    my ($next, $curr, $prev);
    my ($album,$album_id);
    if ($opt{id}) {
	# album passed by id#
	$album_id = $opt{id};
	$album = $db->fetch("page",$album_id);
    }
    elsif ($opt{name}) {
	# album passed by name
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>$opt{name}});
	$album = $lib[0];
	$album_id = $album->{page_id};
    }
    if (!$album) {
	# look for "images" library in current site
        my @lib = $db->fetch_match("page",{section_id=>$share{Page}->id("section"),
                                           filename=>"images"});
	$album = $lib[0];
	$album_id = $album->{page_id};
    }
    my $gwidth = $opt{width} || $config{Gallery}{width} || 4;

    if ($album) { 
	my $out = "<div class=\"Gallery\">
<script langauge=\"JavaScript\" src=\"$config{server}{HTMLpath}/_ExSite/js/misc.js\"></script>\n";
	if (exists $opt{title} ? $opt{title} : 1) { 
	    $out .= "<h1>$album->{title}</h1>\n";
	}
	if ($album->{description}) { 
	    $out .= "<p>$album->{description}</p>\n";
	}
	$out .= "<table class=\"Gallery\"><tr>\n";

	my @pics = $db->fetch_child("content","page",
				    $album_id,"name");

	# display the thumb grid

	my $n = 0;
	foreach my $p (@pics) {
	    my $c = new ExSite::Content(content=>$p);
	    my $descr = $c->get("content","description");
	    $c->set_context("child");
	    my $thumb = $c->get_thumb_html;
	    my $url = $c->get_url;
	    my $attr = $c->get("content_data","attributes");
	    $attr =~ /width=\"?(\d+)\"?/i;
	    my $width = $1+20 || 600;
	    $attr =~ /height=\"?(\d+)\"?/i;
	    my $height = $1+20 || 600;
	    $out .= "<td align=\"center\" class=\"Gallery\" valign=\"top\"><a href=\"javascript:popup_custom($width,$height,'$url')\">$thumb</a><br>
$descr</td>\n";
	    $n++;
	    if (($n % $gwidth) == 0) {
		$out .= "</tr><tr>\n";
	    }
	}
	if ($n % $gwidth != 0) {
	    while ($n % $gwidth != 0) { $n++; $out .= "<td class=\"Gallery\">&nbsp;</td>"; }
	}
	$out .= "</tr></table>\n</div>\n";
	return $out;
    }
    else {
	my $message = "Gallery not found ($options)";
	return $this->error($message);
    }
}

sub ioctl {
    my $this = shift;
    $_ = shift;
    if (/ModuleName/) {
	return "Photo Gallery";
    }
    elsif (/ModuleInfo/) {
	return "The Photo Gallery tool allows your website users to browse
through thumbnails of an image library, and select particular images to 
few at full size.";
    }
}

1;
