package Modules::HomePage;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------
# The HomePage module is just a way of placing a shortcut to the
# website homepage on the WebTop, so that one does not have to 
# navigate through the Website Editor to get a preview.
# The read and write methods are dummies; only the control panel is 
# significant.

use strict;
#use ExSite::Config;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub read {
    # do nothing
    return;
}

sub write {
    # do nothing
    return "";
}

sub ioctl {
    my ($this,$request) = @_;
    if ($request eq "ModuleName") {
	return "Home Page";
    }
    elsif ($request eq "ControlPanel") {
	return \&ctrl_panel;
    }
    elsif ($request eq "ModuleInfo") {
	return "The Home Page tool provides you with a simple shortcut to
jump from your administrator control panel back to the regular public side
of your website.";
    }
}

sub ctrl_panel {
    my $this = shift;
    require ExSite::Auth;
    my $a = new ExSite::Auth;
    my @sites = $a->my_real_sites;
    my $out;
    if (@sites > 1) {
	$out .= "<p>Multiple websites found, please choose one:</p>\n";
    }
    $out .= "<dl>\n";
    require ExSite::Section;
    foreach my $site (@sites) {
	my $s = new ExSite::Section(section=>$site);
	$s->set_context;
	my $section = $s->get;
	my $path = $s->diskpath;
	my $url = $s->get_url;
	if (! -e $path) {
	    $out .= "<dt>$section->{title} (NOT PUBLISHED YET)</dt>\n";
	    $out .= "<dd>$section->{description}</dd>\n";
	} 
	elsif (@sites == 1) {
	    print "Location: $url\n\n";
	    return;
	}
	else {
	    $out .= "<dt><a href=$url>$section->{title}</a></dt>\n";
	    $out .= "<dd>$section->{description}</dd>\n";
	}
    }
    $out .= "</dl>\n";
    return $out;
}


1;
