package Modules::Member;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

# Member : simple toolkit for managing members

### TODO
###
### handler for selecting only those sites with member service

use strict;
use ExSite::Config;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# no read method - this is a dumb module

# write : do nothing - this is a control panel service only

sub write {
    my ($this,$options) = @_;
    my $db = $share{DB};
    my $out;
    if ($db->authorize) {
	my $in = new ExSite::Input;
	my $data = $in->post;
	if (keys %$data > 0) {
	    $out .= $this->update($data);
	}
	elsif ($this->{input}{mbrcmd}) {
	    $_ = $this->{input}{mbrcmd};
	    if (/pass/) {
		return $this->passwd_form;
	    }
	    elsif (/contact/) {
		return $this->contact_form;
	    }
	    else {
		$out .= "<p class=error>Unknown member command: $_</p>\n";
	    }
	}
	my $name = $this->show_data("name");
	my $contact_url = $this->link(mbrcmd=>"contact");
	my $pwd_url = $this->link(mbrcmd=>"pass");
	$out .= "<center>
<h1>$name</h1>
<table cellpadding=50><tr>
<td align=center>
<a href=\"$contact_url\"><img src=\"$config{server}{HTMLpath}/_Modules/Member/contact.gif\" height=100 width=100 border=0><br>
My Contact Info</a></td>
<td align=center>
<a href=\"$pwd_url\"><img src=\"$config{server}{HTMLpath}/_Modules/Member/passwd.gif\" height=100 width=100 border=0><br>
My Password</a></td>
</tr></table>
</center>\n";
    }
    else {
	$out = "<p>You are not logged in</p>\n";
    }
	return $out;
}

# ioctl : supports the webtop integration requests, and nothing else

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    # handle the standard ioctl requests for webtop integration
    if (/ModuleName/) {
	# the user-friendly name for this app
	return "Members";
    }
    elsif (/ControlPanel/) {
	# a reference to the administrator control panel for this app
	return \&ControlPanel;
    }
    elsif (/isRestricted/) {
	return exists $config{Member}{isRestricted} ? $config{Member}{isRestricted} : 0;
    }
    elsif (/ModuleInfo/) {
	return "This is a member-management tool that lets you create, 
delete, and edit the various users who have permission to log in to the
private members-only areas of your website.";
    }
    return undef;
}

sub ControlPanel {
    my $this = shift;
    my $db = $share{DB};
    my $out;
    my $cmd = $this->{input}{mbrcmd};
    my @sites;
    if ($this->{input}{site}) {
	@sites = (scalar $db->fetch("section",$this->{input}{site}));
    }
    else {
	@sites = $db->my_real_sites;
    }

    # administrator options

    if ($db->is_admin) {
	if ($cmd eq "all") {
	    return $db->report_relational({tab=>"member"});
	}
	elsif (@sites > 1) {
	    my $url = $this->link(mbrcmd=>"all",member=>undef);
	    $out .= "<p><a href=$url>All members</a> (system administrator view)</p>\n";
	}
    }

    if ($this->{input}{member}) {
	my $member = $db->fetch("member",$this->{input}{member});
	if ($member) {
	    if (! $db->owns($member->{section_id})) { 
		return "<p class=error>Permission denied.</p>\n";
	    }
	}
	else {
	    return "<p class=error>Permission denied - invalid member ID.</p>\n";
	}
    }

    # hidden fields in member forms
    my @hide;
    if (! $db->is_admin) { @hide = ("access"); }
    if (@sites == 1) { push @hide,"section_id"; }

    # execute member functions

    if ($cmd eq "edit") {
	my $action = $this->link(caladm=>"submit",member=>undef);
	$out .= "<h1>Edit Member</h1>\n<dl>\n";
	$out .= $db->make(table=>"member",
			  record=>$this->{input}{member},
			  action=>$action,
			  hide=>\@hide,
			  extra=>{mbrcmd=>"submit"},
			  );
	return $out;
    }
    elsif ($cmd eq "new") {
	my $action = $this->link(caladm=>"submit",member=>undef);
	$out .= "<h1>New Member</h1>\n<dl>\n";
	$out .= $db->make(table=>"member",
			  action=>$action,
			  hide=>\@hide,
			  data=>{section_id=>$this->{input}{site}},
			  extra=>{mbrcmd=>"submit"},
			  );
	return $out;
    }
    elsif ($cmd eq "submit") {
	$db->get_data;
	if ($db->{submit_action} ne "Cancel") {
	    # tweak the form data
	    my @memid = keys %{$db->{form}{member}};
	    my $memid = $memid[0] || "_";
	    $db->{action} = (exists $db->{form}{member}{$memid}{member_id} ? 
			     "update" : "insert");
	    
	    # validate inputs

	    my @err;
	    if ($db->{form}{member}{$memid}{section_id}) {
		# ownership check
		if (! $db->owns($db->{form}{member}{$memid}{section_id})) {
		    push @err, "Invalid website - permission denied.";
		}
	    }
	    elsif ($db->{action} ne "update") {
		push @err, "Member does not belong to any website.";
	    }
	    my @mem;
	    if ($db->{action} eq "insert") {
		@mem = $db->fetch_match("member",{login=>$db->{form}{member}{$memid}{login}});
	    }
	    else {
		@mem = $db->custom_query("select * from member where login=\"$db->{form}{member}{$memid}{login}\" and member_id != $memid");
	    }
	    
	    if (@mem > 0) {
		push @err, "That login name has been taken - please choose another.";
	    }
	    if (@err > 0) {
		$out .= "<h3>Problem:</h3>\n<p class=\"error\">\n".
		    join("<br>\n",@err)."</p>\n";
		$out .= "<a href=\"javascript:history.back()\">Back to form</a>\n";
		return $out;
	    }
	    
	    $db->{form}{member}{$memid}{access} = 1;
	    # process the form (more validation done here)
	    $db->do;
	    # check errors
	    if ($db->errorcheck) {
		$out = "<H3>Problem:</H3>\n<p>";
		$out .= $db->show_diagnostics("error","html");
		$out .= "</p>\n<P><A HREF=javascript:history.back()>Go back to form.</A></P>\n";
	    }
	}
    }
    elsif ($cmd eq "del") {
	my $m = $db->fetch("member",$this->{input}{member});
	if (! $db->owns($m->{section_id})) {
	    return "<p class=error>Permission denied.</p>\n";
	}
	$out .= "<p>Deleting $m->{name}... ";
	$db->trash_key("member",$this->{input}{member});
	$out .= "done!</p>\n";
    }

    # membership reports

    foreach my $site (@sites) {
	my %report = ( title => "$site->{title} Members",
		       nodata => "No members found.",
		       headers => [ "Name", "Login", "E-mail", "Options" ],
		       data => [] );
	my @members = $db->fetch_child("member","section",$site->{section_id},"name");
	if (@members > 0) {
	    foreach my $m (@members) {
		my @data;
		push @data, ($m->{name},$m->{login},
			     $db->show_data("member","email",$m->{email}));
		my $url = $this->link(mbrcmd=>"edit",member=>$m->{member_id});
		my $opt = "<a href=$url><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{edit}{img}\" height=\"$config{db_ops}{edit}{img_height}\" width=\"$config{db_ops}{edit}{img_width}\" border=\"0\" alt=\"edit\" title=\"edit\"></a> \n";
		my $url = $this->link(mbrcmd=>"del",member=>$m->{member_id});
		$opt .= "<a href=$url><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{delete}{img}\" height=\"$config{db_ops}{delete}{img_height}\" width=\"$config{db_ops}{delete}{img_width}\" border=\"0\" alt=\"delete\" title=\"delete\"></a> \n";
		push @data,$opt;
		push @{$report{data}},\@data;
	    }
	}
	my $url = $this->link(mbrcmd=>"new",,member=>undef,site=>$site->{section_id});
	$report{tools} = "<a href=$url><img src=\"$config{server}{HTMLpath}/_Modules/WebDB/$config{db_ops}{new}{img}\" height=\"$config{db_ops}{new}{img_height}\" width=\"$config{db_ops}{new}{img_width}\" border=\"0\" alt=\"add new member\" title=\"add new member\"></a>";
	my $r = new ExSite::ReportBuilder(%report);
	$out .= "<p>".$r->make."</p>";
    }
    return $out;
}

sub show_data { 
    my ($this,$col) = @_;
    my $db = $share{DB};
    return $db->show_data("member",$col,$share{identity}{lookup}{$col});
}

sub passwd_form {
    my $this = shift;
    my $url = $this->link;
    return "<h3>Change Password</h3>
<form action=$url method=post>
<p>Old password:<br>
<input type=password name=passwd0></p>
<p>New password:<br>
<input type=password name=passwd1><br>
<input type=password name=passwd2> (enter again, to confirm)</p>
<input type=submit value=\" Change Password \">
<input type=button value=\" Cancel \" OnClick=\"javascript:history.back()\">
</form>\n";
}

sub contact_form {
    my $this = shift;
    my $url = $this->link;
    my $m = $share{identity}{lookup};
    return "<h3>Update Contact Info</h3>
<form action=$url method=post>
<p>Name:<br>
<input type=text name=name size=60 value=\"$m->{name}\"></p>
<p>E-mail:<br>
<input type=text name=email size=40 value=\"$m->{email}\"></p>
<input type=submit value=\" Submit \">
<input type=button value=\" Cancel \" OnClick=\"javascript:history.back()\">
</form>\n";
}

sub update {
    my ($this,$in) = @_;
    my $out;
    my $db = $share{DB};
    if ($in->{passwd0}) {
	my $err = $db->change_password($in->{passwd0},$in->{passwd1},$in->{passwd2});
	if ($err) {
	    $out = "<p class=error>$err</p>";
	}
	else {
	    $out = "<p>Password was changed sucessfully.</p>\n";
	}
    }
    elsif (exists $in->{name} || exists $in->{email}) {
	my $out;
	if (! $in->{name}) {
	    return "<p class=error>You must provide a name.</p>\n";
	}
	elsif (! $in->{email}) {
	    $out .= "<p class=error>WARNING: without a defined email address
you may miss important updates and notices.</p>\n";
	}
	my $err = $db->update("member",$in,{member_id=>$db->my_uid});
	if ($err) {
	    $out .= "<p class=error>$err</p>";
	}
    }
    return $out;
}

1;
