package Modules::MySite;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#   MySite is a simplified content management system that is suitable for
#   newbies and ExSite-Lite users.  It inlines all CMS functions into the
#   website itself, rather than the administrator control panel.  That 
#   makes it a little harder to set up, since a website must preexist to 
#   embed the MySite tools into;  but much easier for casual users to use 
#   once the parent site is running.
#
#   Features of MySite:
#       - creates native websites only
#       - creates one library per site (called "images")
#       - simplified library management ("My Image Library")
#       - creates static pages only
#       - inline content editor appears right in page
#       - simplified wizards
#       - one-step content updates
#       - assumes one website per user for simpler navigation
#
#   For advanced CMS features like template building, version control, 
#   polymorphic content, dynamic page configuration, raw-HTML editing, etc. 
#   you must use the CMS module instead.
#----------------------------------------------------------------------------
#
#   TODO
#   - website options to control access to system features
#   - thumbnail previews in image library
#
#----------------------------------------------------------------------------

use strict;
use MIME::Base64;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Util;
use ExSite::Mail;
use ExSite::Input;
use ExSite::Image;
use ExSite::Section;
use Modules::BaseDCD;
use Modules::CMS;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

my $rowcount;

# write : redirect to user control panel, or prompt for login

sub write {
    my ($this,$options) = @_;
    my %odata = &DecodeString($options);
    my $cmd = $odata{ecmd} || $this->{input}{ecmd};
    # public tools
    if ($cmd eq "newsite" && $config{MySite}{allow_public_newsite}) {
	return $this->newsite;
    }
    elsif ($cmd eq "remind") {
	return $this->reminder;
    }
    elsif ($cmd eq "menu") {
	return "";  # deprecated function
    }
    # member-only tools
    if ($share{DB}->authorize()) {
	return $this->my_switchboard($options);
    }
    # login
    return $this->Authenticate($options);
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "My Website";
    }
    elsif (/ControlPanel/) {
	#my @sites = $share{DB}->my_real_sites;
	#if (@sites > 1) {
	#    # too many sites - redirect to CMS instead
	#    return "ctrl-panel.cgi/CMS";
	#}
	# only one site, so we can use MySite Control Panel
	return \&my_switchboard;
    }
    elsif (/ModuleInfo/) {
	return "My Website is a simple, user-friendly website management
tool.  It doesn't support the full range of features of the Website 
Editor, but makes all of the commonly-used features easier to use.
My Website lets you easily create and delete pages, edit your pages,
change your design templates, and upload and manage a library of photos
or other images.";
    }
    return undef;
}

sub mysite {
    my $this = shift;
    if (! $this->{site}) {
	my $db = $share{DB};
	my %u = $db->my_user_record;
	if ($u{section_id}) {
	    $this->{site} = $db->fetch("section",$u{section_id});
	}
	else {
	    my @sites = $share{DB}->my_real_sites;
	    $this->{site} = $sites[0];
	}
    }
    return $this->{site};
}

sub myurl {
    my $this = shift;
    my $site = $this->mysite;
    return $site->{url} ?
	$site->{url} :
	"$config{server}{server}$config{server}{HTMLpath}/$site->{publish_to}";
}

# user's control panel

sub my_switchboard {
    my ($this,$options) = @_;
    my $db = $share{DB};
    my $site = $this->mysite;
    if (! $site) {
	return "Permission denied - no website found.";
    }
    my $in = new ExSite::Input;
    my $data = $in->combine();
    my %odata = &DecodeString($options);

    # special handling of inlined editing tools

    if ($odata{ecmd} eq "edit_content_link") {
	if ($data->{ecmd} eq "update") {
	    # replace the edit link with an editor
	    return $this->update_body;
	}
	elsif ($data->{ecmd} eq "doupdate") {
	    return $this->do_update;
	}
	else {
	    # insert a link to edit content inline
	    my $url = &relink(ecmd=>"update");
	    return "<br><a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/b_edit.gif\" height=16 width=64 border=0 alt=\"Edit this page\" style=\"padding:2px;\"></a>";
	}
    }

    # finally, user-selected options

    if ($data->{submit_action} eq "Cancel") {
	return $this->my_ctrl($site);
    }

    my $cmd = $data->{ecmd} || $odata{ecmd};
    if ($cmd eq "edit") {
	if ($share{Page}) {
	    $share{Page}->redirect("$config{server}{CGIpath}/home.cgi");
	}
	elsif ($share{PageBuf}) {
	    $share{PageBuf}->location("$config{server}{CGIpath}/home.cgi");
	}
    }
    elsif ($cmd eq "newsite" && $config{MySite}{allow_private_newsite}) {
	return $this->newsite;
    }
    elsif ($cmd eq "summary") {
	return $this->site_summary;
    }
    elsif ($cmd eq "newpage") {
	return $this->newpage;
    }
    elsif ($cmd eq "pageconf") {
	return $this->pageconf;
    }
    elsif ($cmd eq "updatepage") {
	return $this->update_page;
    }
    elsif ($cmd eq "delpage") {
	return $this->delpage;
    }
    elsif ($cmd eq "rollback") {
	return $this->rollback;
    }
    elsif ($cmd eq "template") {
	return $this->template;
    }
    elsif ($cmd eq "config") {
	return $this->config;
    }
    elsif ($cmd eq "passwd") {
	return $this->passwd;
    }
    elsif ($cmd eq "images") {
	return $this->library;
    }
    elsif ($cmd eq "addimg") {
	return $this->image_add;
    }
    elsif ($cmd eq "delimg") {
	return $this->image_remove;
    }
    elsif ($cmd eq "updateimg") {
	return $this->image_update;
    }
    return $this->my_ctrl($site);
}

# mysite style sheet

sub mysite_css {
#    if (! $share{Page}) {
	return "<style>
table.MySiteMap {
  border:1px #ccccdd solid; 
  border-collapse:collapse; 
}
table.MySiteMap td,th {
  border:1px #ccccdd solid;
  margin:0px;
  padding:5px;
}
table.MySiteMap th {
  background-image:url($config{server}{HTMLpath}/_Modules/MySite/rfiller.gif);
  background-color:#d0d0f0;
  padding:7px;
}
table.MySiteMap tr.a {
  background-color:#ffffff;
}
table.MySiteMap tr.b {
  background-color:#f7f7ff;
}


table.MySiteMapIcons {
  border:0px solid white;
  padding:0px;
}
table.MySiteMapIcons td,th {
  border:0px white solid;
  margin:0px;
  padding:0px;
}

table.MySiteMenu {
  border:1px #ccccdd solid; 
  border-collapse:collapse; 
}
table.MySiteMenu td,th {
  border:1px #ccccdd solid;
  margin:0px;
  padding:5px;
}
table.MySiteMenu th {
  background-image:url($config{server}{HTMLpath}/_Modules/MySite/rfiller.gif);
  background-color:#d0d0f0;
  padding:7px;
}
table.MySiteMenu td {
  background-color:#f7f7ff;
  padding:5px;
}
table.MySiteMenu a {
  text-decoration:none;
  display:block;
}
table.MySiteMenu td:hover {
  background-color:f0f0ff;
}
</style>\n";
#    }
}

# my control panel

sub my_ctrl {
    my $this = shift;
    my $site = shift || $this->mysite;
    my $db = $share{DB};
    my $out = "<h1>$site->{title}</h1>
<script LANGUAGE=javascript>\nfunction popup_link(url) {\n  window.open(url,\"\",\"width=600,height=400,resizable,scrollbars=1,menubar=0\")\n}\n</SCRIPT>\n";
    my $mysite_files = "$config{server}{HTMLpath}/_Modules/MySite";
    $out .= $this->mysite_css;
    $out .= "<table cellspacing=0 cellpadding=0 width=100% border=0>
<tr><td valign=top>
<table class=\"MySiteMap\" width=540>
<tr><th>Web Page</th><th>Filename</th><th><div id=\"opt\">Options <a href=javascript:ShowOpts() style=\"text-decoration:none\">[+]</a></div></th></tr>\n";

    # get top-level pages

    my %match = (section_id=>$site->{section_id},
		 type=>"page",
		 status=>"active",
		 visibility=>"visible",
		 parent_id=>0,);
    my @pages = $db->fetch_match("page",\%match,["rank","page_id"]);
    $rowcount = 0;

    # track sitemap info for javascript
    $this->{divs} = [];
    $this->{contents} = [];
    foreach my $page (@pages) {
	$out .= $this->sitemap($page,0);
    }

    $out .= "<tr><th colspan=3>Special Pages and Libraries</th></tr>\n";

    # hidden pages

    %match = (section_id=>$site->{section_id},
		 type=>"page",
		 status=>"active",
		 visibility=>"hidden");
    @pages = $db->fetch_match("page",\%match,["rank","page_id"]);
    foreach my $page (@pages) {
	$out .= $this->sitemap_row($page,0);
    }

    # libraries

    %match = (section_id=>$site->{section_id},
		 type=>"library",
		 status=>"active");
    @pages = $db->fetch_match("page",\%match,["rank","page_id"]);
    foreach my $page (@pages) {
	$out .= $this->sitemap_row($page,0);
    }

    $out .= "</table>\n";

    # menu

    $out .= "</td><td width=10><img src=$config{server}{HTMLpath}/_ExSite/images/null.gif height=10 width=10></td><td valign=top>\n";

    $out .= "<table class=MySiteMenu width=100%>
<tr><th>Other Functions</th></tr>\n";
    my $url = $this->link(ecmd=>"newpage");
    $out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_blank.gif\" border=\"0\" align=\"middle\"> New Page</a></td></tr>\n";
    $url = $this->link(ecmd=>"template");
    $out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_theme.gif\" border=\"0\" align=\"middle\"> Set Theme</a></td></tr>\n";
    $url = "$config{server}{CGIpath}/publish.cgi?section=$site->{section_id}";
    $out .= "<tr><td><a href=\"javascript:popup_link('$url')\"><img src=\"$mysite_files/ico_pub.gif\" border=\"0\" align=\"middle\"> Publish Site</a></td></tr>\n";
    $url = $this->link(ecmd=>"config");
    $out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_conf.gif\" border=\"0\" align=\"middle\"> Configure Site</a></td></tr>\n";
    $url = $this->link(ecmd=>"passwd");
    $out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_pass.gif\" border=\"0\" align=\"middle\"> Change Password</a></td></tr>\n";
    if ($this->site_type eq "advanced") {
	$url = "$config{server}{CGIpath}/home.cgi";
	$out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_adv.gif\" border=\"0\" align=\"middle\"> Advanced Control Panel</a></td></tr>\n";
    }
    #$url = "http://exsitewebware.com/support/mysite.html";
    $url = "javascript:popup_link('$config{server}{CGIpath}/help.cgi/_Modules/MySite/help.html')";
    $out .= "<tr><td><a href=$url><img src=\"$mysite_files/ico_help.gif\" border=\"0\" align=\"middle\"> Help</a></td></tr>\n";
    $url = "$config{server}{CGIpath}/logout.cgi";
    my $myurl = $this->myurl;
    my $target = $config{MySite}{public_url_target} ?
	" target=\"$config{MySite}{public_url_target}\"" :
	"";
    $out .= "<tr><td><a href=\"$url\"><img src=\"$mysite_files/ico_logout.gif\" border=\"0\" align=\"middle\"> Logout</a></td></tr>
</td></tr></table>
</div>

<p><img src=\"$mysite_files/caution20.gif\" height=\"20\" width=\"20\" align=\"middle\"> You must publish your site before your changes will be visible to the public!</p>
<p>The public URL to your website is: <a href=\"$myurl\"$target>$myurl</a></p>
</td><tr></table>\n";

    # build javascript for dynamic icon panels

    my $ndiv = scalar @{$this->{divs}};
    my $js = "<script language=javascript>
var divs = new Array(\n'".
    join("',\n'",@{$this->{divs}}).
    "')
var contents = new Array(\n'".
    join("',\n'",@{$this->{contents}}).
    "')
function HideOpts() {
  var locopt = document.getElementById(\"opt\")
  locopt.innerHTML = 'Options <a href=javascript:ShowOpts() style=\"text-decoration:none\">[+]</a>'
  for (i = 0; i < divs.length; i++) {
    var loc = document.getElementById(divs[i])
    loc.innerHTML = ''
  }
}
function ShowOpts() {
  var locopt = document.getElementById(\"opt\")
  locopt.innerHTML = 'Options <a href=javascript:HideOpts() style=\"text-decoration:none\">[-]</a>'
  for (i = 0; i < divs.length; i++) {
    var loc = document.getElementById(divs[i])
    loc.innerHTML = contents[i]
  }
}
</script>\n";

    return "$js$out";
}

sub sitemap {
    my ($this,$page,$depth) = @_;
    # recursive mapping routine
    my $db = $share{DB};
    my $pid = $page->{page_id};
    my $out .= $this->sitemap_row($page,$depth);
    my $subpage = $db->fetch_match("page",{parent_id=>$pid,
					   type=>"page",
					   visibility=>"visible"},
				   ["rank","page_id"]);
    if (@$subpage > 0) {
	foreach my $page (@$subpage) { 
	    $out .= $this->sitemap($page,$depth+1);
	}
    }
    return $out;
}

sub sitemap_row {
    my ($this,$page,$depth) = @_;
    $rowcount++;
    # add divname to list of dynamic divs
    push @{$this->{divs}}, "r".$rowcount;
    my $dyndiv;
    my $db = $share{DB};
    my $url;
    my $out = "<tr class=".($rowcount % 2 ? "a" : "b").">\n  <td>";
    if ($depth > 1) {
	$out .= "<img src=\"$config{server}{HTMLpath}/_ExSite/images/null.gif\" height=\"1\" width=\"".(($depth-1)*20)."\" border=\"0\">";
    }
    if ($depth) {
	$out .= "<img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_drop.gif\" height=\"20\" width=\"20\" border=\"0\" align=\"middle\">";
    }
    if ($page->{type} eq "library") { 
	$out .= "Library: "; 
    }
    elsif ($page->{type} eq "page" && $page->{visibility} eq "hidden") {
	$out .= "Hidden page: ";
    }
    $out .= $page->{label} || $page->{title};
    if ($page->{type} eq "page" && $page->{access} ne "public") {
	$out .= " <i>[RESTRICTED]</i>";
    }
    $out .= "</td>\n  <td>";
    if (($page->{publish_method} eq "static" &&
	$page->{access} eq "public") || 
	$page->{type} ne "page") {
	$out .= $page->{filename};
    }
    elsif ($page->{access} eq "members") {
	$out .= "<i>[members-only page]</i>";
    }
    else {
	$out .= "<i>[dynamic page]</i>";
    }
    $out .= "</td>\n  <td><table class=\"MySiteMapIcons\" cellspacing=0 cellpadding=0 border=0><tr><td>";
    if ($page->{type} eq "page") {
	# preview
	$out .= "<a href=\"$config{server}{CGIpath}/page.cgi?_id=$page->{page_id}&_cms=1\" target=\"preview\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_page.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"preview\" title=\"preview\"></a> ";
	# edit
	$out .= "<a href=\"$config{server}{CGIpath}/page.cgi?_id=$page->{page_id}&_cms=1&ecmd=update\" target=\"preview\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_edit.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"edit\" title=\"edit\"></a> ";
    }
    elsif ($page->{type} eq "library" && $page->{filename} eq "images") {
	# edit
	$url = $this->link(ecmd=>"images",id=>undef);
	$out .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_edit.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"update\" title=\"update\"></a> ";
    }
    $out .= "</td><td><div id=\"r$rowcount\"></div></td></tr></table>";
    # publish
    $dyndiv .= "<a href=\"$config{server}{CGIpath}/publish.cgi?page=$page->{page_id}\" target=\"publish\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_pub.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"publish\" title=\"publish\"></a> ";
    if ($this->site_type eq "advanced") {
	# advanced CMS shortcut
	$url = "$config{server}{CGIpath}/ctrl-panel.cgi/CMS?path=/$page->{section_id}/$page->{page_id}";
	$dyndiv .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_padv.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"advanced page builder\" title=\"advanced page builder\"></a> ";
    }
    if ($page->{type} eq "page") {
	# config
	$url = $this->link(ecmd=>"pageconf",id=>$page->{page_id});
	$dyndiv .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_pconf.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"configure\" title=\"configure\"></a> ";
	# rollback
	$url = $this->link(ecmd=>"rollback",id=>$page->{page_id});
	$dyndiv .= "<a href=\"javascript:popup_link(\\'$url\\')\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_rollback.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"rollback\" title=\"rollback\"></a> ";
    }
    # delete
    if ($page->{type} eq "page" && 
	$page->{filename} !~ /^index.(php|asp|html?)$/) {
	$url = $this->link(ecmd=>"delpage",id=>$page->{page_id});
	$dyndiv .= "<a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_del.gif\" border=\"0\" height=\"20\" width=\"20\" alt=\"delete\" title=\"delete\"></a> ";
    }
    # add dyndiv contents to contents list
    push @{$this->{contents}}, $dyndiv;
    $out .= "</td>\n<tr>\n";
    return $out;
}

sub Authenticate {
    my ($this,$options) = @_;
    if ($this->{input}->{login}) {
	# attempted login
	# clear last attempt to identity user
	delete $share{identity};
	my $in = $this->{input};
	require ExSite::Auth;
	my $a = new ExSite::Auth;
	if ($a->login($in->{login},$in->{password})) {
	    if ($this->mysite) {
		return $this->my_ctrl($options);
	    }
	    else {
		my $out = "<h3 class=\"error\">Login Failed</h3>
<p>Please double-check your login ID and password.</p>";
		$out .= $this->LoginForm();
		return $out;
	    }
	}
	else {
	    my $out = "<h3 class=\"error\">Login Failed</h3>
<p>Please double-check your login ID and password.</p>";
	    $out .= $this->LoginForm();
	    return $out;
	}
    }
    return $this->LoginForm();
}

sub LoginForm {
    my $this = shift;
    my $url = &relink();
    my $remindurl = &relink(ecmd=>"remind");
    return "<center><br><br>
<div>
<h1>My Site</h1>
<form method=\"post\" action=\"$url\">
<table class=\"form\">
<tbody><tr><td>Login:</td><td><input type=\"text\" name=\"login\"></td></tr>
<tr><td>Password:</td><td><input type=\"password\" name=\"password\"></td></tr>
</tbody></table>
<p><input type=\"submit\" value=\" Log in \"></p>
</form>
<br><br>
<p><a href=\"$remindurl\">Please send me my login and password again.</a></p>
</div>
<br><br><br></center>
\n";
}

sub reminder {
    my $this = shift;
    my $reminder_sent;
    my $out = "<br><br>\n<h1>My Site (Password Reminder)</h1>\n";
    if ($this->{input}{siteid} || $this->{input}{email}) {
	my (@member,%match);
	if ($this->{input}{siteid}) {
	    $match{login} = $this->{input}{siteid};
	}
	if ($this->{input}{email}) {
	    $match{email} = $this->{input}{email};
	}
	@member = $share{DB}->fetch_match("member",\%match);
	if (@member > 0) {
	    my $info;
	    if (@member > 1) {
		$info = "(You have several sites at ExsiteWebware.com.  Each of the following
Login/Password pairs accesses one site.)\n\n";
	    }
	    foreach my $m (@member) {
		$info .= "Login:    $m->{login}
Password: $m->{password}\n\n";
	    }
	    &SendMail($member[0]->{email},$config{site}{administrator},
		      "ExSite WebWare login reminder",
		      "This message was sent to you because someone requested a login/password 
reminder at ExSite WebWare.  If that person was not you, don't worry 
because only you are receiving this message.

Here is your login info:		      

$info
-- The ExSite Webware Team\n");
	    my $login = &relink(ecmd=>undef,action=>undef);
	    $out .= "<p>Password reminder sent.  It should arrive in your inbox soon.</p>\n<p><a href=\"$login\">Retry login</a></p>";
	    $reminder_sent = 1;
	}
	else {
	    $out .= "<p class=\"error\">No matching websites were found.</p>\n";
	}
    }
    if (! $reminder_sent) {
	my $url = &relink(ecmd=>"remind");
	$out .= "<p>Please provide us with either your login ID, or your email address.</p>
<form method=\"post\" action=\"$url\">
<table class=\"form\">
<tbody><tr><td>Site ID:</td><td><input type=\"text\" name=\"siteid\"></td></tr>
<tr><td>OR, your email address:</td><td><input type=\"email\" name=\"email\"></td></tr>
</tbody></table>
<p><input type=\"submit\" value=\" Submit \"></p>
</form>\n";
    }
    $out .= "<br><br>\n";
    return $out;
}

sub newsite {
    my $this = shift;
    my $action = &relink();
    my $wiz = new ExSite::Wizard(title=>"New Website",(action=>$action));
    $wiz->add_step("Identification",\&wiz_easy_section_page1,\&do_wiz_easy_section_page1);
    $wiz->add_step("Website Type",\&wiz_easy_section_page2,\&do_wiz_easy_section_page2);
    $wiz->add_step("Pick a Theme",\&wiz_easy_section_page3,\&do_wiz_easy_section_page3);
    $wiz->add_step("Instructions",\&wiz_easy_section_page4,\&do_wiz_easy_section_page4);
    return $wiz->go();
}

sub newpage {
    my $this = shift;
    my %page;
    my $db = $share{DB};
    my $out;
    my $site = $this->mysite(); 
    my @pages = $db->fetch_match("page",{type=>"page",
					 section_id=>$site->{section_id},
				     });
    my $limit = $this->page_limit;
    # assume that page_limit == 0 means no limit
    if ($limit && scalar @pages >= $limit) {
	$out = "<h1>Page Limit Exceeded</h1>
<p>The present configuration of your site allows for $limit pages.  
You will need to upgrade your site's capacity before you can add more pages.
Alternatively, you could remove other pages that you no longer need.</p>\n";
    }
    elsif ($this->{input}{'label'}) {
	# we have input
	$this->insert_page;
    }
    else {
	# no input - throw up a form
	my $action = &relink();
	my $out .= "<h1>Make a new web page</h1>
<p>Your web page will appear in site menus under a
short but descriptive name.  Please specify the menu label to use for
this new page:</p>
<form METHOD=\"POST\" ACTION=\"$action\">
<span CLASS=\"form\">
<p><span CLASS=\"label\">Menu Label</span><br>
<input type=\"text\" size=\"40\" name=\"label\" value=\"New Page\"></p>
</span>
<p><input TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;
<input TYPE=reset>&nbsp;&nbsp;
</p>
</form>
<p><b>Note</b>: To customize the details or position of this page after you have created 
it, use the configure page function.  Click on the [+] link by \"Options\", 
which will reveal the configure page button: 
<img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_pconf.gif\" height=\"20\" width\"20\" border=\"0\" align=\"middle\"></p>\n";
	return $out;
    }
}

# reconfigure a page

sub pageconf {
    my $this = shift;
    if ($this->{input}{label}) {
	# we have input
	return $this->update_page;
    }
    my $pid = shift || $this->{input}{id};
    my $db = $share{DB};
    my %page;
    if (! $pid) { return "<p class=error>No page to configure!</p>\n"; } 
    %page = $db->fetch("page",$pid);
    my $action = &relink();
    my $out = "<h2>Configure Page</h2>
<form METHOD=\"POST\" ACTION=\"$action\">
<span CLASS=\"form\">
<p><span CLASS=\"label\">Menu Label</span><br>
<input type=\"text\" size=\"40\" name=\"label\" value=\"$page{label}\"></p>
<p><span CLASS=\"label\">Title - defaults to menu label if not set</span><br><input type=\"text\" size=\"60\" name=\"title\" value=\"$page{title}\"></p>
<p><span CLASS=\"label\">Description - may be used for search engine optimization.</span><br><input type=\"text\" size=\"60\" name=\"description\" value=\"$page{description}\"></p>
<p><span CLASS=\"label\">Keywords - may be used for search engine optimization.</span><br><input type=\"text\" size=\"60\" name=\"keywords\" value=\"$page{keywords}\"></p>
<p><span CLASS=\"label\">Parent Page - sets the position in the site map.</span><br>\n";

    $out .= "<select name=\"parent_id\"><option value=\"\" selected>== select parent page ==</option>";
    my $site = $this->mysite;
    my @pages = $db->fetch_match("page",{type=>"page",
					 section_id=>$site->{section_id},
				     });
    foreach my $p (@pages) {
	next if ($p->{page_id} == $pid);
	$out .= "<option value=\"$p->{page_id}\""; 
	if ($page{parent_id} == $p->{page_id}) { 
	    $out .= " selected"; 
	}
	$out .= ">$p->{title}</option>";
    }
    $out .= "</select></p>\n";

    $out .= "<p><span CLASS=\"label\">Page Rank - a number (typically 1 - 9) used for ordering pages in menus and submenus (lower values appear higher in menus).</span><br>
<input type=\"text\" size=\"4\" name=\"rank\" value=\"$page{rank}\"></p>
<p><span CLASS=\"label\">Privacy</span><br><input type=\"radio\" name=\"access\" value=\"public\"";
    if ($page{access} eq "public") { $out .= " checked"; }
    $out .= "> Public page <input type=\"radio\" name=\"access\" value=\"members\"";
    if ($page{access} eq "members") { $out .= " checked"; }
    $out .= "> Members only</p>";
    $out =~ s/\'/\&\#39/g;
    $out .= "<input type=\"hidden\" name=\"page_id\" value=\"$pid\">\n";
    $out .= "<p><input TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;
<input TYPE=submit NAME=submit_action VALUE=\"Cancel\">&nbsp;&nbsp;
<input TYPE=reset>&nbsp;&nbsp;
</p>
</form>\n";
    return $out;
}

sub update_page {
    my $this = shift;
    my $db = $share{DB};
    my $site = $this->mysite;
    my @err;
    my $oldpage = $share{DB}->fetch("page",$this->{input}{id});
    if ($oldpage->{section_id} ne $site->{section_id}) {
	push @err, "site mismatch";
    }
    # we have input - process the form
    my $label = $this->{input}{'label'};
    my $title = $this->{input}{'title'} || $label;
    my $pid;
    my %page = (
		page_id=>$this->{input}{page_id},
		label=>$label,
		title=>$title,
		access=>($this->{input}{access} || "public"),
		rank=>$this->{input}{rank},
		description=>$this->{input}{description},
		keywords=>$this->{input}{keywords},
		parent_id=> ($this->{input}{parent_id} ? $this->{input}{parent_id} : 0),
		section_id=>$site->{section_id},
		);
    
    my $err = $db->update("page",\%page);
    if ($err) { push @err, $err; }
    if (@err > 0) {
	return "<h4 class=error>".join("<br>\n",@err)."</h3>\n";
    }
    return $this->my_ctrl;
}

sub insert_page {
    my $this = shift;
    my $db = $share{DB};
    my $site = $this->mysite;
    # we have input - process the form
    my $label = $this->{input}{'label'};
    my $title = $this->{input}{'title'} || $label;
    my @err;
    my $pid;
    my %page = (
		label=>$label,
		title=>$title,
		type=>"page",
		status=>"active",
		access=>($this->{input}{access} || "public"),
		publish_method=>"static",
		visibility=>"visible",
		rank=>($this->{input}{rank} || "1"),
		description=>$this->{input}{description},
		keywords=>$this->{input}{keywords},
		parent_id=> ($this->{input}{parent_id} ? $this->{input}{parent_id} : 0),
		section_id=>$site->{section_id},
		template_id=>$site->{template_id},
		);
    my $filename = $label;
    $filename =~ s/[\W]/_/g;  # convert bad chars to _

    # check for existing pages
    my $n = undef;
    my @exist = $db->fetch_match("page",{section_id=>$site->{section_id},
					 filename=>$filename."$n.html"});
    while (@exist > 0) {
	$n++;
	@exist = $db->fetch_match("page",{section_id=>$site->{section_id},
					  filename=>$filename."$n.html"});
    }
	
    $page{filename} = $filename."$n.html";
	
    $pid = $db->insert("page",\%page);
    if (! $pid) {
	push @err, ("failed to add new page: ".
		    $db->show_diagnostics("error","html"));
    }
    else {
	# now create a body
	my %content = (
		       page_id => $pid,
		       name => "body",
		       type => "editorial",
		       );
	my $id = $db->insert("content",\%content);
	if (! $id) {
	    push @err, ("page was created, but failed to add page body: ".
			$db->show_diagnostics("error","html"));
	}
    }
	
    my $out;
    if (@err > 0) {
	$out .= "<h1>Error:</h1>\n<p class=\"error\">".
	    join("<br>\n",@err)."</p>\n";
	$out .= "<p><form><input type=button value=\"&lt; Back\" onclick=\"history.back()\"></form></p>\n";
    }
    else {
	$out .= $this->my_ctrl;
    }
    return $out;
}

sub update_body {
    my $this = shift;
    my $db = $share{DB};
    my ($out, %content, @cdata, $selected, $id, $pid);

    # get the page id and content id from the current page object
    my $id;
    if ($share{Page}) {
	$pid = $share{Page}->id();
	if ($pid) {
	    my @content = $share{Page}->fetch_match("content",{name=>"body",
							       page_id=>$pid});
	    if (@content > 0) {
		%content = %{$content[-1]};
		$id = $content[-1]->{content_id};
	    }
	    else {
		my $err = "no page body found";
		$this->error($err);
		return "<p class=\"error\">Error: $err</p>\n";
	    }
	}
	else {
	    my $err = "undefined page";
	    $this->error($err);
	    return "<p class=\"error\">Error: $err</p>\n";
	}
    }
    else {
	my $err = "MySite::update_body can only be used inlined into a page";
	$this->error($err);
	return "<p class=\"error\">Error: $err</p>\n";
    }

    my $action = &relink(ecmd=>"doupdate");
    $out .= "<p><img src=\"$config{server}{HTMLpath}/_ExSite/images/caution20.gif\" height=\"20\" width=\"20\" border=\"0\" align=\"middle\"> You must hit \"Submit\" (at bottom) to save your changes.  If you navigate away from this page
using the menu links, your updates will be discarded.</p>\n";
    $out .= "<form METHOD=POST action=\"$action\" onSubmit=\"validate()\">\n";

    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#data#html");
    $out .= $db->input_html(type=>"hidden",name=>"action",value=>"insert");
    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#mime_type",value=>"text/html");

    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#content_id", value=>$id);

    if ($id =~ /\d+/) {
	#%content = $db->fetch("content",$id);
	@cdata = $db->fetch_child("content_data","content",$id,"content_data_id");
    }
    if (@cdata == 0) {
	$cdata[0] = { content_data_id=>"_2", mime_type => "text/html" };
    }
    my $cid = $cdata[-1]{content_data_id};
    my $name = $content{name};
    $out .= "<input type=hidden name=\"content_data_id\" value=\"$cid\">\n"; 

    my $update_method;
    if ($cdata[-1]->{mime_type} =~ /^text\/html$/o) {
	$update_method = "html";
    }
    else {
	$update_method = "upload";
    }

    my $textdata;
    if ($db->use_wysiwyg) {
	my $type = $this->site_type;
	my $op_image = $config{MySite}{$type}{allow_addimg};
	my $op_dyncontent = $config{MySite}{$type}{allow_dynamic};
	$out .= $db->input_htmleditor(name=>"content_data#_2#data",
				      table=>"content_data",
				      record=>$cid,
				      page=>$pid,
				      #section=>$this->mysite()->{section_id},
				      section=>$share{Page}->id("section"),
				      column=>"data",
				      value=>"$textdata",
				      editor=>{
					  op_image=>$op_image,
					  op_link=>1,
					  op_table=>1,
					  op_dyncontent=>$op_dyncontent,
				      });
	$out .= $db->editor_javascript;
    }
    else {
	# default to text area
	$out .= "<p class=\"error\">WARNING: Your browser is not compatible
with the Exsite Webware editor.  To use the editor, you should upgrade to
one of the following browsers:  Windows IE 6.0+, Netscape Navigator 7.1+, 
Mozilla 1.4+, or Firefox.  Otherwise, you are restricted to editing in 
raw HTML mode:</p>\n";
	$out .= "<p><textarea cols=80 rows=24 wrap=virtual name=\"content_data#_2#data\">\n";
	$out .= $textdata;
	$out .= "</textarea></p>\n";
    }

    # 3) revision control comment

    my $comment = "updated by ".$db->my_name." on ".scalar(localtime);
    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#note",value=>$comment);

    $out .= "</p>\n<INPUT TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;\n<INPUT TYPE=submit NAME=submit_action VALUE=\"Cancel\">&nbsp;&nbsp;\n";

    return $out;
}

sub do_update {
    my $this = shift;
    my $form = $share{DB};
    $form->get_data;

    # patch up those mangled page links
    $form->{form}->{content_data}->{_2}->{data} =~ s/%7B/{/g;
    $form->{form}->{content_data}->{_2}->{data} =~ s/%7D/}/g;

    # restore escaped objects such as images, DCDs
    # REMOVED - THIS IS DONE ELSEWHERE
    #$form->{form}->{content_data}->{_2}->{data} = 
    #	&restore_cms_tags($form->{form}->{content_data}->{_2}->{data});

    $form->do_db("insert");
    if ($share{Page}) {
	$share{Page}->location("page.cgi?_id=$this->{input}{_id}&_cms=2");
    }
    else {
	return "Page updated.";
    }
}

sub delpage {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($this->{input}{id}) {
	# specific page id given in URL
	my $page = $this->{input}{id};
	my $pdata = $db->fetch("page",$page);
	if ($this->{input}{confirm}) {
	    $out .= "deleting page $page...<br>";
	    if ($db->approve("delete","page",$page)) {
		$db->trash_r("page",$page);
	    }
	    &redirect($this->link("ecmd"=>undef,"confirm"=>undef,"id"=>undef));
	}
	else {
	    my $action = $share{Page} ? "page.cgi" : "ctrl-panel.cgi/MySite";
	    $out .= "<h4>Delete page \"$pdata->{title}\"</h4>
<p>This action will remove this page from your website.  Your website
should be republished after doing this, so that the menus on other pages
will be updated appropriately.</p>
<p>If you change your mind, deleted pages can be restored from the
Trash in the advanced control panel.</p>
<p><form method=\"get\" action=\"$config{server}{CGIpath}/$action\">
<input type=hidden name=\"_id\" value=\"$this->{input}{_id}\">
<input type=hidden name=\"id\" value=\"$this->{input}{id}\">
<input type=hidden name=\"ecmd\" value=\"$this->{input}{ecmd}\">
<input type=hidden name=\"confirm\" value=\"1\">
<input type=submit value=\" DELETE \">
<input type=button value=\" Cancel \" OnClick=\"javascript:history.back()\"></form></p>\n";
	}
    }
    else {
	$out = "<h1>Error Deleting Page</h1>\n<p>No page specified!</p>\n";
    }
    return $out;
}

sub rollback {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($this->{input}{id}) {
	my $c = new ExSite::Content(page=>$this->{input}{id},name=>"body");
	$c->rollback;
    }
    if ($share{Page}) {
	$share{Page}->errorpage("
<html><body>
<center>
<h3>Rollback completed</h3>
<p><a href=\"javascript:window.close()\">Close Window</a></p>
</center>
</body></html>\n");
    }
    else {
	return "<center>
<h3>Rollback completed</h3>
<p><a href=\"javascript:window.close()\">Close Window</a></p>
</center>\n";
    }
}

sub template {
    my $this = shift;
    my $out;
    my $db = $share{DB};
    if ($ENV{REQUEST_METHOD} eq "POST") {
	my $tid = $this->{input}{template};
	if ($tid) {
	    # 1) set site's default template
	    my $site = $this->mysite;
	    $db->update("section",{template_id=>$tid},{section_id=>$site->{section_id}});
	    # 2) reset template id of all pages in this site
	    $db->update("page",{template_id=>$tid},{section_id=>$site->{section_id}, type=>"page"});
	    $out .= "<H3>Done!</H3>\n";
	    $out .= "<p>This change will not be visible until you republish your site.</p>\n";
	}
	else {
	    $out .= $this->Error("No template selected.");
	}	    
	my $url = $this->link(ecmd=>undef);
	$out .= "<P><A HREF=$url>Back to My Site</A></P>\n";
    }
    else {
	# no form data - publish a form
	$out = "<h1>New Template</h1>\n<form method=\"POST\">\n";
	my $site = $this->mysite;
	$out .= $this->list_template($site->{template_id});
	$out .= "<p><input type=\"submit\" value=\" Submit \">
<input type=\"button\" value=\" Cancel \" OnClick=\"javascript:history.back()\">
</p>\n";
    }
    return $out;
}

# simplified version of CMS::list_template

sub list_template {
    my ($this,$id) = @_;
    my ($out,@ptempl,@options);
    my $site = $this->mysite()->{section_id};
    $out .= "<p>Select a template:</p>\n<blockquote>\n";

    # a little javascript to control the target window

    $out .= "<script LANGUAGE=\"javascript\">
<!--
function NewWindow(target){
	var NewWin = window.open(target,'','width=800,height=500,scrollbars,resizable')
	}
-->
</script>\n";

    my $db = $share{DB};

    # fetch public templates
    @ptempl = $db->fetch_match("page",{type=>"template",access=>"public"});
    # ... plus the member-only templates from the current section
    push @ptempl, $db->fetch_match("page",{type=>"template",access=>"members",section_id=>$site});
    my $sec = $db->fetch("section",$site);
    if ($sec && $sec->{parent_id}) {
	# ... plus the member-only templates from the parent section
	push @ptempl, $db->fetch_match("page",{type=>"template",access=>"members",section_id=>$sec->{parent_id}});
    }
    # ...plus the private templates from this section
    push @ptempl, $db->fetch_match("page",{type=>"template",access=>"administrators",section_id=>$site});

    if (@ptempl > 0) {
	foreach my $pt (sort { lc($a->{title}) cmp lc($b->{title}) } @ptempl) {
	    $out .= "<p><dl><dt><input type=\"radio\" name=\"template\" value=\"$pt->{page_id}\"";
	    $out .= ($pt->{page_id} == $id) ? " checked" : "";
	    $out .= "> <b>$pt->{title}</b></dt>\n";
	    my $preview_url = "javascript:NewWindow('$config{server}{CGIpath}/page.cgi?_id=$pt->{page_id}')";
	    $out .= "<dd>$pt->{description}  [<A HREF=\"$preview_url\">preview</A>]</dd></dl>\n";
	}
    }
    else {
	$out .= "<p>Sorry, there don't seem to be any templates to choose from.</p>\n";
    }
    $out .= "</blockquote>\n";
    return $out;
}


sub site_summary {
    my $this = shift;
    my $site = $this->mysite;
    my $out .= "<table width=100%>\n";
    foreach my $key ("title","description","publish_to","template_id") {
	$out .= "<tr><th>".
	    $share{DB}{map}->get_column_attr("section",$key,"label").
	    "</th></tr>\n<tr><td>".
	    $share{DB}->show_data("section",$key,$site->{$key}).
	    "</td></tr>\n";
    }
    $out .= "</table>\n";
    return $out;
}

sub config {
    my $this = shift;
    my $site = $this->mysite;
    my $out;
    if ($this->{input}{title}) {
	# form input present - process the input

	# sanity checks ??

	$share{DB}->update("section",{section_id=>$site->{section_id},
				      title=>$this->{input}{title},
				      description=>$this->{input}{subtitle}});
	my $url = $this->link(ecmd=>undef);
	$out = "<h1>Website Reconfigured</h1>
<p>Changes to titles and descriptions will not be publicly visible until
you re-publish the site.</p>
<p><a href=$url>Back to control panel</a>.</p>\n";
    }
    else {
	$out = "<h1>My Website Configuration</h1>\n";
	my $url = &relink();

	##### escape quotes etc. in field values?

	$out .= "<form method=\"POST\" action=\"$url\">
<div class=\"form\">
<p>Website Title:<br>
<input type=\"text\" name=\"title\" size=\"60\" value=\"$site->{title}\"></p>
<p>Subtitle/Description:<br>
<input type=\"text\" name=\"subtitle\" size=\"60\" value=\"$site->{description}\"></p>
</div>
<p><input type=\"submit\" value=\"Submit\">
<input type=\"reset\" value=\"Reset\"></p>
</form>\n";
    }
    return $out;
}

sub passwd {
    my $this = shift;
    my $out;
    if ($this->{input}{passwd1}) {
	# form input present - process the input
	my $stat = $share{DB}->change_password($this->{input}{passwd0},
					       $this->{input}{passwd1},
					       $this->{input}{passwd2});
	if ($stat) { return $stat; }  # error
	my $url = $this->link(ecmd=>undef);
	$out .= "<h3>Password Changed Successfully</h3>
<p><a href=$url>Back to website control panel</a>.</p>\n";
    }
    else {
	$out = "<h1>Change My Password</h1>\n";
	my $url = &relink();
	$out .= "<form method=\"POST\" action=\"$url\">
<div class=\"form\">
<p>Current Password: <br>
<input type=\"password\" name=\"passwd0\"></p>
<p>New Password: <br>
<input type=\"password\" name=\"passwd1\"><br>
(enter it again, to confirm): <br>
<input type=\"password\" name=\"passwd2\"></p>
</div>
<p><input type=\"submit\" value=\"Submit\"></p>
</form>\n";
    }
    return $out;
}

sub publish {
    my ($this,$page) = @_;
    my $db = $share{DB};
    my $site = $this->mysite;
    my $out;
    if ($page) {
	my $p = $db->fetch("page",$page);
	if ($p->{section_id} == $site->{section_id}) {
	    # spawn the publish program, so it runs setuid
	    $ENV{QUERY_STRING} = "page=$page";
	    $out .= `$config{CGIroot}/publish.cgi`;
	}
	else {
	    $out .= "<p class=\"error\">Permission denied: invalid page.</p>\n";
	}
    }
    else {
	# spawn the publish program, so it runs setuid
	$ENV{QUERY_STRING} = "section=$site->{section_id}";
	$out .= `$config{server}{CGIroot}/publish.cgi`;
    }
    return $out;
}

# find the default library for the site

sub my_library {
    my ($this) = @_;
    my $db = $share{DB};
    my $site = $this->mysite();
    my $lib = $db->fetch_match("library",{section_id=>$site->{section_id},
					  filename=>"images"});
    return $lib->[0];
}

# display the library contents

sub library {
    my ($this) = @_;
    my $lib = $this->my_library();
    if (! $lib) {
	return $this->Error("No image library found.");
	# if we want to be nice, we should include an option to create one
    }

    my @contents = $share{DB}->fetch_child("content","page",$lib->{page_id});

    my $out .= "<h2>My Image Library</h2>
<p>Click on the image name to preview it.</p>
<script LANGUAGE=javascript>\nfunction popup_link(url) {\n  window.open(url,\"\",\"width=400,height=400,resizable,scrollbars=1,menubar=0\")\n}\n</SCRIPT>\n";
    $out .= $this->mysite_css;

    my $libout;
    my $nimg = 0;
    foreach my $c (@contents) {
	# foreach :
	# show : name, size
	# options : view, update, delete
	$nimg++;
	my $content = new ExSite::Content(content=>$c);
	$content->set_context("child");
	my $cdata = $content->{context}{content_data};
	my $size = int(length($cdata->{fdata})*6/8/100)/10;
	my $viewurl = "javascript:popup_link('$config{server}{CGIpath}/viewcontent.cgi/$c->{name}?id=$cdata->{content_data_id}')";
	$libout .= "<tr class=\"".
	    ($nimg % 2 ? "A" : "B").
	    "\"><td><a href=\"$viewurl\">$c->{name}</a></td><td>$size K</td><td>";
	my $delurl = &relink(ecmd=>"delimg",image=>$c->{content_id});
	my $updurl = &relink(ecmd=>"updateimg",image=>$c->{content_id});
	$libout .= "<a href=\"$updurl\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_img.gif\" height=20 width=20 border=\"0\" title=\"Update Image\" alt=\"Update Image\"></a>
<a href=\"$delurl\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_del.gif\" height=20 width=20 border=\"0\" title=\"Delete Image\" alt=\"Delete Image\"></a></td></tr>\n";
    }
    if ($libout) {
	$out .= "<table class=MySiteMap>\n<tr><th>Name</th><th>Size</th><th>Options</th></tr>\n".
	    $libout."</table>\n";
	# kludge : the background color is set here because the exsitewebware 
	# site's css is flaky.  This may violate other site's style rules.
    }
    else {
	$out .= "<p>No images have been uploaded yet.</p>\n";
    }
    
    # add new image

    my $url;
    if (! $config{MySite}{$this->site_type}{allow_addimg}) {
	$url = "javascript:alert('You must upgrade your site to upload images.')";
    }
    else {
	my $imglimit = $this->image_limit;
	if ($imglimit == 0 || (scalar @contents < $imglimit)) {
	    $url = &relink(ecmd=>"addimg");
	}
	else {
	    $url = "javascript:alert('You are only allowed $imglimit uploaded images.')";
	}
    }
    $out .= "<p><a href=\"$url\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/ico_newimg.gif\" border=\"0\" alt=\"Add Image\" align=\"middle\">Add New Image</a></p>\n";
    $url = $this->link(ecmd=>undef);
    $out .= "<p>&laquo; <a href=\"$url\">Back to website control panel</a>.</p>\n";

    return $out;
}

sub image_add {
    my $this = shift;
    my $out;
    my $lib = $this->my_library();
    if (! $lib) {
	return $this->Error("No image library found.");
	# if we want to be nice, we should include an option to create one
    }
    if (! $config{MySite}{$this->site_type}{allow_addimg}) {
	$out .= $this->Error("You must upgrade your site to upload images.");
    }
    else {
	my @contents = $share{DB}->fetch_child("content","page",$lib->{page_id});
	my $imglimit = $this->image_limit;
	if ($imglimit == 0 || (scalar @contents < $imglimit)) {
	    $out .= $this->image_upload;
	}
	else {
	    $out .= $this->Error("You are only allowed $imglimit uploaded images.");
	}
    }
    return $out;
}

sub image_update {
    my $this = shift;
    return $this->image_upload($this->{input}{image});
}

sub image_upload {
    my ($this,$image_id) = @_;
    my ($img,$content_id,$in,$data,$err,$first,$out,$lib);
    $in = new ExSite::Input;
    $data = $in->post();
    $first = 1;

    if ($data->{upload}) {
	# user is uploading an image

	# check for image id
	my $img = $data->{image};
	if ($img) {
	    # looks like an update
	    my $content = $share{DB}->fetch("content",$img);
	    $lib = $this->my_library();
	    if ($content->{page_id} == $lib->{page_id}) {
		$content_id = $content->{content_id};
	    }
	    else {
		$err = "Invalid image $data->{image} $img specified $content->{page_id} != $lib->{page_id}";
	    }
	}

	# perform the update
	if (! $err) {
	    $first = 0;

	    # read file contents
	    my $rawdata = undef;
	    my $size = 0;
	    my $mimedata = "";
	    my $file = $data->{upload};
	    my $continue = 1;
	    while ($continue) {
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	    }

	    # check for a valid file
	    if ($file !~ /\.(png|gif|jpe?g)$/i) {
		# wrong file type
		$err = "Incorrect file type.  ExSite will accept only
GIF, PNG, or JPEG images.  If your images are of another type, you will
need to use a photo manipulation program to convert the file type to one
of these.";
	    }
	    elsif ($size == 0) { 
		# no file data
		$err = "No file data was received.";
	    }
	    else {
		# we seem to have a valid file

		# strip path from filename
		my @path = split /[:\\\/]/, $file;
		my $filename;
		if ($path[-1]) { $filename = &clean_filename($path[-1]); }
		else { $filename = "image".time(); }
		my $img = new ExSite::Image($filename,$rawdata);
		my $max_img_size = $config{MySite}{max_image_size} || 64;
		if ($size > ($max_img_size * 1024)) {
		    # file size too big
		    $img->shrink(size => $max_img_size);
		    $size = $img->size;
		}
		my ($x,$y) = $img->dim;

		# ok - save the file
		my $lib = $this->my_library();
		# MIME-encode the file contents
		my $fdata = $img->encode;
		if (! $content_id) {
		    # this is a new image - create the content object first
		    $content_id = $share{DB}->insert("content",{page_id=>$lib->{page_id},
								name=>$filename,
								type=>"editorial",}
						     );
		}
		if ($content_id) {
		    my $comment = "uploaded on ".(scalar localtime);
		    my ($x,$y) = $img->dim;
		    my $attr = "width=$x height=$y";
		    my ($thumb,$th_attr);
		    if ($img->thumb < 49000) {
			# only thumb if thumbnail is < 48K
			$thumb = "th_".$img->encode;
			($x,$y) = $img->dim;
			$th_attr = "width=$x height=$y";
		    }
		    if (! $share{DB}->insert("content_data",{content_id=>$content_id,
							     mime_type=>&MimeType($file),
							     note=>$comment,
							     fdata=>$fdata,
							     attributes=>$attr,
							     thumb=>$thumb,
							     thumb_attributes=>$th_attr,
							 })) {
			$err = "Error saving uploaded file.";
		    }
		}
		else {
		    $err = "Error configuring uploaded file.";
		}
	    }
	}
    }
    if ($err) {
	$out = $this->Error($err);
    }
    if ($first || $err) {
	# show an upload form
	$out .= "<h2>Upload a new image</h2>
<form method=\"POST\" enctype=\"multipart/form-data\">
<input type=\"hidden\" name=\"image\" value=\"$image_id\">
<p>Select image file: <input type=\"file\" name=\"upload\">&nbsp;&nbsp;
<input type=\"submit\" value=\"Upload\"></p>
</form>

<h3>Tips</h3>
<ul>
<li> Only GIF, JPEG, and PNG files will be accepted.</li>
<li> Note that JPEG images are best for photographs.  GIF or PNG may
work better for line art.  If your image is is another format, you will
need to convert it to one of these formats using a graphic editing 
program.</li>
<li> If you require your image to have precise dimensions, you will
need to resize the image first, using a graphic editing program.
Otherwise it will be automatically resized to be no larger than 500
pixels across and 64K in size.</li>
</ul>
";
    }
    else {
	# successful upload - back to library view
	$out .= $this->library;
    }
    return $out;
}

sub image_remove {
    my $this = shift;
    my $db = $share{DB};
    $this->{input}{image} =~ /^(\d+)$/;
    my $img = $1;
    my $out;
    if ($img) {
	my $content = $db->fetch("content",$img);
	if ($content->{page_id} == $this->my_library()->{page_id}) {
	    $db->trash_r("content",$img,1);
	}
	else { 
	    $out = $this->Error("Invalid image specified.");
	}
    }
    else { 
	$out = $this->Error("No such image.");
    }
    $out .= $this->library;
    return $out;
}

# update : update the content on the current page

sub update {
    my $this = shift;
    my $db = $share{DB};
    my ($out, %content, @cdata, $selected);
    
    if ($this->{input}{reply}) {
	# we have input - try to perform the update
	$db->get_data;
	
	# patch up those mangled page links
	$db->{form}->{content_data}->{_2}->{data} =~ s/%7B/\{/g;
	$db->{form}->{content_data}->{_2}->{data} =~ s/%7D/\}/g;

	$db->do_db("insert");
	my $reply = $this->{input}->{reply} || "$config{server}{CGIpath}/page.cgi?_id=$this->{page}";
	if ($share{Page}) {
	    $share{Page}->redirect($reply);
	}
	elsif ($share{PageBuf}) {
	    $share{PageBuf}->location($reply);
	}
    }
    else {
	# no input - prompt for some
	my $reply;
	if ($ENV{HTTP_REFERER} =~ /page\.cgi/) {
	    $reply = $ENV{HTTP_REFERER};
	}
	else {
	    $reply = "page.cgi?_id=$this->{page}";
	}
	my $action = &relink(cmscmd=>"doupdate");
	$out = $db->editor_javascript;
	$out .= "<form METHOD=POST action=\"$action\" onSubmit=\"validate()\">\n";
	
	$out .= $db->input_html(type=>"hidden",name=>"content_data#_2#data#html");
	$out .= $db->input_html(type=>"hidden",name=>"action",value=>"insert");
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
	$out .= $db->input_html(type=>"hidden",name=>"content_data#_2#mime_type",value=>"text/html");
	
	# get the content id from the query data
	my $id = $this->{content};
	
	$out .= $db->input_html(type=>"hidden",name=>"content_data#_2#content_id", value=>$id);
	
	if ($id =~ /\d+/) {
	    %content = $db->fetch("content",$id);
	    @cdata = $db->fetch_child("content_data","content",$id,"content_data_id");
	}
	if (@cdata == 0) {
	    # set an assumed mime type, depending on content type
	    if ($content{type} eq "design") {
		# also check for content#_1#allow_mime#_set_N = "image"
		$cdata[0] = { content_data_id=>"_2", mime_type => "image/gif" };
	    }
	    else {
		$cdata[0] = { content_data_id=>"_2", mime_type => "text/html" };
	    }
	}
	my $cid = $cdata[-1]{content_data_id};
	my $name = $content{name};
	$out .= "<input type=hidden name=\"content_data_id\" value=\"$cid\">\n"; 
	
	my $update_method;
	if ($cdata[-1]->{mime_type} =~ /^text\/html$/o) {
	    $update_method = "html";
	}
	else {
	    $update_method = "upload";
	}
	
	my $textdata;
	if ($update_method =~ /html$/ && $db->use_wysiwyg) {
	    $out .= $db->input_htmleditor(name=>"content_data#_2#data",
					  table=>"content_data",
					  record=>$cid,
					  page=>$this->{page},
					  section=>$this->{section},
					  column=>"data",
					  value=>"$textdata",
					  editor=>{
					      op_image=>0,
					      op_link=>1,
					      op_table=>1,
					      op_dyncontent=>0,
					  });
	    
	}
	elsif ($update_method =~ /^upload$/) {
	    $out .= "<p>Click on browse to select a file to upload:</p>\n";
	    $out .= "<p><input type=file size=60 name=\"content_data#_2#fdata#filename\"></p>\n";
	    $this->{db}->{enctype} = "multipart/form-data";
	    ######################## need a multipart-encoded form!
	    ########## should be done by Form
	}
	else {
	    # default to text area
	    $out .= "<p>You may enter your content changes directly into this text editing area:</p>\n";
	    $out .= "<p><textarea cols=80 rows=20 wrap=virtual name=\"content_data#_2#data\">\n";
	    $out .= $textdata;
	    $out .= "</textarea></p>\n";
	}
	
	# 3) revision control comment
	
	my $comment = "updated by ".$db->my_name." on ".scalar(localtime)."\n";
	$out .= $db->input_html(type=>"hidden",name=>"content_data#_2#note",value=>$comment);
	
	$out .= "</p>\n<INPUT TYPE=submit NAME=submit_action VALUE=\"Submit\">&nbsp;&nbsp;\n<INPUT TYPE=submit NAME=submit_action VALUE=\"Cancel\">&nbsp;&nbsp;\n";
	return $out;
    }
}

# -------- easy site creation wizard
#
# assumes: native site, new webmaster

sub wiz_easy_section_page1 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out;

    # set reply page on first screen
    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || &relink(cmscmd=>undef);
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }

    $out = "<p>First, provide us with a valid e-mail address.  This is needed
to communicate with you on the status of your website, send password and login
instructions, and so on.  It is kept strictly confidential.</p>
<p>Email address:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"email");

    $out .= "<p>Next, give us the full name or title of your site.  For
example: <b>$config{MySite}{sample_website_name}</b>
<p>Full Website Name:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"title", size=>60);

    $out .= "<p>You can optionally provide a longer website description.
Some graphic designs may display this information as a subtitle, while 
others will simply use it as a hidden description for search engines. 
For example: <b>$config{MySite}{sample_website_description}</b>
<p>Description/subtitle (optional):<br>\n";
    $out .= $db->input_html(type=>"text", name=>"description", size=>60);

    $out .= "<p>You also need to define a short version of the website
name.  This must be a short text string with no spaces or punctuation.
It will be used in your
website URL, and also as a login ID for you to access your site and
edit it.  You may use up to 32 letters, numerals, underscores, and
hyphens.  For example: <b>$config{MySite}{sample_website_id}</b></p>
<p>Short Website ID:<br>\n";
    $out .= $db->input_html(type=>"text", name=>"sitename");
    $out .= "</p>\n";
    return $out;
}

sub do_wiz_easy_section_page1 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err;
    if ($in->{"email"} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	push @err, "Invalid e-mail address.";
    }
    if ($in->{"sitename"} !~ /^[A-Za-z0-9_\-]{3,32}$/) {
	push @err, "Invalid website ID.  The site ID must be between
3 and 32 characters long, containing only letters, numerals, underscores, 
and hyphens.";
    }
    if (! $in->{"title"}) {
	# not an error
	$in->{title} = $in->{sitename};
    }

    if ($in->{"sitename"}) {
	my @match = $db->fetch_match("section",{publish_to=>$in->{sitename}});
	if (@match > 0) {
	    push @err, "Sorry, a website with that ID already exists in the system.  Please try another.";
	}
	else {
	    @match = $db->fetch_match("member",{login=>$in->{sitename}});
	    if (@match > 0) {
		push @err, "Sorry, a website with that ID already exists in the system.  Please try another.";
	    }
	}
    }

    return @err;
}

sub wiz_easy_section_page2 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my $out;

    $out .= "<p>Choose your website type.  This determines the pages that
will be created for you automatically.</p>\n";

    my @wtempl = $db->fetch_match("section","type=\"website template\" and (parent_id=0 or parent_id is NULL)");

    if (@wtempl > 0) {
	$out .= "<p><dl>";
	foreach my $wt (@wtempl) {
	    $out .= "<dt>".
		$db->input_html(type=>"radio",name=>"website_template",
				value=>$wt->{section_id},
				label=>"<b>$wt->{title}</b></dt>\n<dd>$wt->{description}</dd>\n");
	}
	$out .= "</dl></p>\n";
    }
    
    return $out;
}

sub do_wiz_easy_section_page2 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_easy_section_page1($this,$in);
    if (! $in->{website_template}) {
	push @err, "Please select a website type!";
    }
    return @err;
}

sub wiz_easy_section_page3 {
    my $this = shift;
    my $in = shift;
    my $out;
    my $db = $share{DB};
    $out .= "<p>Now you can select the visual theme of your site, from the
options provided below.  Use the <b>preview</b> links to view samples of
each design.</p>\n";
    require Modules::CMS;
    my $cms = new Modules::CMS();
    $out .= $cms->list_template("page_template",undef,2);
    return $out;
}

sub do_wiz_easy_section_page3 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_easy_section_page2($this,$in);
    if (! $in->{page_template}) {
	push @err, "Please select a graphic design template!";
    }
    return @err;
}

sub wiz_easy_section_page4 {
    my $this = shift;
    my $in = shift;
    return "<p>Thank You! We have collected all the information we need to create
your website.  When you click on \"FINISH\", below, your website will be
configured for you.  Instructions for how to log in to manage your 
website will be emailed to <font color=#0000FF><tt>$in->{email}</tt></font>.
</p><p>
After you receive this email, click on the \"My Site\" button at the top.
You can enter your login information there to access your website
administration tools.
</p>\n";
}

sub do_wiz_easy_section_page4 {
    my $this = shift;
    my $in = shift;
    my $db = $share{DB};
    my @err = &do_wiz_easy_section_page3($this,$in);
    return @err if (@err > 0);

    # if we get here, we're ready to go

    # we need to create:
    # 1) a user
    # 2) a site (section)
    # 3) pages and content objects
    # 4) then mail the login info to the user

    my %section = (title=>$in->{title},
		   type=>"native website",
		   description=>$in->{description},
		   publish_to=>$in->{sitename},
		   note=>"basic",
		   template_id=>$in->{page_template});

    # add website
    my $sid = $db->insert("section",\%section);
    if (! $sid) {
	return ("failed to create new website!");
    }

    my $pwd = &randtext(7,"ABCDEFGHIJKLMNPQRSTUVWXYZ123456789.-");
    my %user = (login=>$in->{sitename},
		password=>$pwd,
		name=>$in->{sitename},
		email=>$in->{email},
		access=>2,
		section_id=>$sid);

    # add user
    my $uid = $db->insert("member",\%user);
    if (! $uid) {
	# undo section insert
	$db->delete("section",$sid);
	return ("failed to create new user account!");
    }

    # add administrator ticket
    my $tid = $db->insert("member_site_link",{section_id=>$sid,
					      member_id=>$uid,
					      priveleges=>"editor; administrator"});
    if (! $tid) {
	return ("failed to create new administrator ticket!");
    }

    # now create the default pages
    if ($in->{website_template}) {
	my @pages = $db->fetch_match("page",{section_id=>$in->{website_template}});
	if (@pages > 0) {
	    foreach my $page (@pages) {
		my %newpage = %$page;
		delete $newpage{page_id};  # force an insert
		$newpage{template_id} = $in->{page_template};
		$newpage{section_id} = $sid;
		my $pid = $db->insert("page",\%newpage);
		if (! $pid) {
		    push @err, "failed to add page \"$page->{title}\"";
		}
		else {
		    # make copies of all editorial content objects
		    my @content = $db->fetch_match("content",{page_id=>$page->{page_id},type=>"editorial"});
		    foreach my $cont (@content) {
			delete $cont->{content_id};  # force an insert
			$cont->{page_id} = $pid;
			my $cid = $db->insert("content",$cont);
			if (! $cid) {
			    push @err, "failed to add \"$cont->{name}\" content to page \"$page->{title}\"";
			}
			else {
			    # copy current content version

			    my @cdata = $db->fetch_match("content_data",{content_id=>$cid},"content_data_id");
			    if (@cdata > 0) {
				my $cdata = $cdata[-1];
				delete $cdata->{content_data_id};  # force an insert
				$cdata->{content_id} = $cid;
				my $cdid = $db->insert("content_data",$cdata);
				if (! $cdid) {
				    push @err, "failed to add \"$cont->{name}\" content data";
				}
			    }
			}
		    }
		}
	    }
	}
    }
   
    # insert an image library, but set its content limit to 0
    my $pid = $db->insert("page",{type=>"library",status=>"active",
				  title=>"image",filename=>"images",
				  section_id=>$sid,});
    
    if (! @err) {
	# we made it!
	# send the user their login info

	require ExSite::Mail;

	my $msg = "This message has been sent to you because someone created
a new website at $config{server}{server}$config{server}{HTMLpath} and provided 
this email address for contact information.

To access the administrator functions for this website, direct your
browser to $config{server}{server}$config{server}{HTMLpath} and click on the 
'My Site' link.

Your login information:

Login ID: $in->{sitename}
Password: $pwd

Enjoy your new website!
";
	# mail to user
	&ExSite::Mail::SendMail($in->{email},$config{site}{administrator},
		  "Your Exsite login information",
		  $msg);
	# notify administrator
	&ExSite::Mail::SendMail($config{site}{administrator},$config{site}{administrator},
		  "ExsiteWebware account created",
		  "The website $in->{sitename} was created by $in->{email}");
    }
			  
    return @err;
}

# site_type : basic, regular, advanced

sub site_type {
    my $this = shift;
    my $note = $this->mysite()->{note};
    return "regular" if ($note =~ /regular/);
    return "advanced" if ($note =~ /advanced/);
    return "basic";
}

sub page_limit {
    my $this = shift;
    return $config{MySite}{$this->site_type}{page_limit};
}

sub image_limit {
    my $this = shift;
    return $config{MySite}{$this->site_type}{image_limit};
}

# capital-E error, so we don't overload the base class error handler

sub Error {
    my ($this) = shift;
    my $out = "<h3 class=\"error\">Problem:</h3>\n";
    foreach my $err (@_) {
	$out .= "<p>$err</p>\n";	
	$this->error($err); # for logging
    }
    return $out;
}

### DEPRECATED?

sub get_option { 
    my ($this,$key) = @_;
    my $site = $this->mysite;
    my @options = split(/\n/,$site->{note});
    my @match = grep(/^$key=/,@options);
    # take the last setting, if there are more than one
    if (@match > 0) {
	my (undef,$value) = split(/=/,$match[-1]);
	return $value;
    }
    return 0;
}

sub set_option { 
    my ($this,$key,$value) = @_;
    my $site = $this->mysite;
    my @options = split(/\n/,$site->{note});
    my @nomatch = grep(!/^$key=/,@options);
    my $newoption = join("\n",@nomatch)."\n$key=$value";
    $site->{note} = $newoption;
    return $share{DB}->update("section",$site);
}

1;
