package Modules::PrintFriendly;
#-----------------------------------------------------------------------
#
#   PrintFriendly : re-render the page in a printer-friendly format.
#
#   This module hacks the context for the page to change its template
#   to a bare-bones template that only includes a body object.
#
#-----------------------------------------------------------------------
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Content;
use ExSite::Input;
use ExSite::Misc;
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$opt) = @_;
    my $p = $share{Page};
    my $db = $share{DB};
    my $in = $this->{input};
    my %opt = &DecodeString($opt);
    $this->{opt} = \%opt;
    my $out;
    if ($in->{printer_friendly}) {
	if (! $this->{printer_friendly}) {
	    my $p = $share{Page};
	    my $site = $p->get("section","title");
	    my $title = $p->get("page","title");
	    #my $url = $share{Page}->get_url;
	    my $url = "http://".$ENV{SERVER_NAME}.$ENV{REQUEST_URI};

	    # hack the layout of the current page, to eliminate the template

	    $share{Page}->{html} = "
<html>
<head>
<title>$title</title>
</head>
<body>
<table align=\"right\"><tr><td style=\"font-size:9pt;\">
<a href=\"javascript:print()\"><img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif\" height=\"16\" width\"16\" border=\"0\" align=\"middle\"> print</a><br>
<a href=\"javascript:window.close()\"><img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/close.gif\" height=\"16\" width\"16\" border=\"0\" align=\"middle\"> close</a>
</td></tr></table>
<h3>$site: $title</h3>
<p>URL: <a href=\"$url\">$url</a></p>
<hr>
<!--content(body)-->
</body>
</html>\n";

	    $this->{printer_friendly} = 1;   # only rebuild the page once
	    $share{Page}{expand_redo} = 1;   # rebuild the page
	}
    }
    else {
	my $url = $this->link(printer_friendly=>1);
	$out = "<a href=\"$url\" target=\"print\">";

	if ($this->{opt}{fmt} eq "icon-on-top") {
	    $out .= "<img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif\" height=16 width=16 border=0><br>
<span style=\"font-size:8pt;\">Printer-friendly version</span>";
	}
	elsif ($this->{opt}{fmt} eq "icon-on-right") {
	    $out .= "<span style=\"font-size:8pt;\">Printer-friendly&nbsp;version</span>&nbsp;<img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif\" height=16 width=16 border=0>";
	}
	elsif ($this->{opt}{fmt} eq "noicon") {
	    $out .= "<span style=\"font-size:8pt;\">Printer-friendly version</span>";
	}
	elsif ($this->{opt}{fmt} eq "icon-only") {
	    $out .= "<img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif\" height=16 width=16 border=0>";
	}
	else {   # icon-on-left
	    $out .= "<img src=\"$config{server}{HTMLpath}/_Modules/PrintFriendly/print.gif\" height=16 width=16 border=0>&nbsp;<span style=\"font-size:8pt;\">Printer-friendly&nbsp;version</span>";
	}
	$out .= "</a>";

	if ($this->{opt}{align} eq "right") {
	    $out = "<table class=PrintFriendly align=right><tr><td align=center>\n$out\n</td></tr></table>\n";
	}
	elsif ($this->{opt}{fmt} eq "icon-on-top") {
	    $out = "<table class=PrintFriendly><tr><td align=center>\n$out\n</td></tr></table>\n";
	}
	else {
	    $out = "<div class=PrintFriendly>\n$out\n</div>\n";
	}
    }
    return $out;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "PrintFriendly regenerates the current page, but in a printer-friendly format with no graphics and minimal colors.";
    }
    elsif (/DynContentParameter/) {
	return "<p><b>Layout:</b>
<select name=\"param\">
<option value=\"\">(default)</option>
<option value=\"fmt=icon-on-left\">Icon then text</option>
<option value=\"fmt=icon-on-right\">Text then icon</option>
<option value=\"fmt=icon-on-top\">Icon above text</option>
<option value=\"fmt=icon-only\">Icon only</option>
<option value=\"fmt=noicon\">No icon (text only)</option>
</select></p>\n";

	### FIXME: removed params with '&align=right' because the & is
	### caught by the browser and escaped, fouling up the formatting
	### of our options string.

    }
    return undef;
}

1;
