package Modules::Publish;
#----------------------------------------------------------------------------
#   The Publish module is a user-friendly interface to publish.cgi;
#   it also is a shortcut to the publish functions in the content management
#   modules such as Website Manager and MySite.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::ContentBase;

# inherit from BaseDCD class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# The Publish tool has no public output

sub write {
    my ($this,$options) = @_;
    return;
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleInfo/) {
	return "The Publish plug-in provides a simple interface to ExSite's publishing tools.";
    }
    elsif (/ControlPanel/) {
	return \&ControlPanel;
    }
}

sub ControlPanel {
    my $this = shift;
    my $db = $share{DB};
    my $out = "<h1>Exsite Publisher</h1>\n";

    # sites

    $out .= "<table cellspacing=20><tr>\n";
    my $nicon;
    foreach my $site ($db->my_real_sites) {
	$nicon++;
	my $url = "$config{server}{CGIpath}/publish.cgi?section=$site->{section_id}";
	$out .= "<td align=center><a href=\"$url\" target=\"publish\"><img src=$config{server}{HTMLpath}/_Modules/CMS/site.gif height=48 width=48 border=0><br>$site->{title}</a></td>\n"; 
	if ($nicon % 4 == 0) { $out .= "</tr><tr>\n"; }
    }

    # modules

    my $c = new ExSite::ContentBase;
    foreach my $mod ($c->get_dcd) {
	if (eval("require Modules::$mod")) {
	    my $m = eval("new Modules::$mod");
	    if ($m) {
		if (ref $m->ioctl("Publish") eq "CODE") {
		    $nicon++;
		    my $modname = $m->ioctl('ModuleName') || $mod;
		    my $icon = $m->icon;
		    my $url = "$config{server}{CGIpath}/publish.cgi?module=$mod";
		    $out .= "<td align=center><a href=\"$url\" target=\"publish\"><img src=\"$icon\" height=\"64\" width=\"64\" border=\"0\"><br>$modname</a></td>\n"; 
		    if ($nicon % 4 == 0) { $out .= "</tr><tr>\n"; }
		}
	    }
	}
    }
    $out .= "</tr></table>
<p>Click on a website (folder) icon, or a web application icon to publish
its files.</p>\n";
    return $out;
}


1;

