package Modules::Security;

# Security : general-purpose security tool
#
# - manages administrator accounts
# - manages general users accounts
# - manages site services and module subscriptions

# list websites with adminstrators, users, services


use strict;
use ExSite::Config;
use ExSite::Form;
use ExSite::Section;
use ExSite::ContentBase;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

# write is a null method;  all functions are accessible only through the 
# administrator webtop

sub write {
    my ($this,$options) = @_;
    return "";
}

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Website Security Manager";
    }
    elsif (/ModuleInfo/) {
	return "The Website Security Manager controls the various security features that 
define who can access websites, and which web applications those sites
are permitted to run.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
}

sub ctrl_panel {
    my $this = shift;
    my $db = $share{DB};
    # only the system administrator is allowed to run this module
    return "<p class=error>Permission denied.</p>" if (! $db->is_admin);

    my $in = $this->{input};
    # <img src=\"$config{server}{HTMLpath}/_Modules/Security/icon.gif\" height=64 width=64 align=left>

    my $out = "<style>
table.SecurityItem {
    border:1px solid #666666;
    border-collapse:collapse; 
}
table.SecurityItem th {
     background-color:#808099;
     border:1px solid #666666;
}
table.SecurityItem td {
     vertical-align:top;
     background-color:#ddddee;
     border:1px solid #666666;
}
table.SecurityItem td.icon {
    width:72;
    background-color: #9999cc;
}
.WarningBox {
    background-color:#FFCCCC;
    border:1px solid red;
    width:500px;
    padding:10px;
}

</style>\n";


    if ($in->{section_id}) {
	my %site = $db->fetch("section",$in->{section_id});
	$out .= "<h1>$site{title}</h1>\n";
    }

    if ($in->{seccmd}) {
	$out .= $this->do_action;
    }
    elsif ($in->{section_id}) {
	$out .= $this->show_site;
    }
    else {
	$out .= $this->list();
    }
    return $out;
}

sub list {
    my $this = shift;
    my $db = $share{DB};
    my $out .= "<h1>Security Manager</h1>
<p>Select a website to review its administrator accounts, general user
accounts, and site services.</p>\n";

    my %report = (
		  title=>"Click on a website name to modify its security settings",
		  headers=>["Website","Administrator Tickets","Users","Services"],
		  data=>[],
		  #foot=>"Select a website name to modify that site's security settings.",
		  );

    my @site = $db->fetch_all("section","section_id");
	
    foreach my $site (@site) {
	next if ($site->{type} =~ /template/);
	my $url = $this->link(section_id=>$site->{section_id});
	my $nticket = $db->count("member_site_link",{section_id=>$site->{section_id}});
	my $nuser = $db->count($config{auth}{user_table},{section_id=>$site->{section_id}});
	my $nserv = $db->count("service",{section_id=>$site->{section_id}});
	
	push @{$report{data}},["<dl><dt><a href=\"$url\">$site->{title}</a></dt>\n<dd>$site->{description}</dd>\n</dl>\n", $nticket, $nuser, $nserv];
    }
    
    my $r = new ExSite::ReportBuilder(%report);
    $out .= $r->make;
    return $out;
}

sub show_site {
    my $this = shift;
    my $db = $share{DB};
    my $sid = $this->{input}{section_id};
    my %site = $db->fetch("section",$sid);
    my $url = $this->link(seccmd=>"delsite");
    my $out = "<p><a href=\"$url\">Delete this site</a>.</p>\n";
    
    # administrator keys

    my @keys = $db->fetch_child("member_site_link","section",$sid);
    $out .= "<table class=\"SecurityItem\" width=500 cellspacing=0 cellpadding=4>
<tr><th colspan=2>Administrator Keys</th></tr>\n";
    foreach my $key (@keys) {
	my ($name, $login);
	my $member = $db->fetch($config{auth}{user_table},$key->{member_id});
	if ($member) {
	    $name = $member->{name};
	    $login = $member->{login};
	}
	else {
	    $name = "undefined member";
	    $login = "NO LOGIN";
	}
	my $upd_url = $this->link(seccmd=>"edkey",key=>$key->{member_site_link_id});
	my $del_url = $this->link(seccmd=>"delkey",key=>$key->{member_site_link_id});
	$out .= "<tr><td class=\"icon\"><img src=\"$config{server}{HTMLpath}/_Modules/Security/key.gif\"></td><td>Administrator Key for <b>$name</b> (login: $login)<br>
This key authorizes the user to act in these capacities:<br>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$key->{priveleges}<p>
<a href=\"$upd_url\">update</a> | <a href=\"$del_url\">delete</a>
</td></tr>\n";
    }
    $url = $this->link(seccmd=>"newkey");
    $out .= "</table><a href=$url>Add New Administrator Key</a><br><br>\n";

    # add/remove services

    $out .= "<table class=\"SecurityItem\" width=500 cellspacing=0 cellpadding=4>
<tr><th colspan=2>Website Services</th></tr>\n";
    my $c = new ExSite::ContentBase;
    my @dcd = $c->get_dcd_all;
    my @services = $db->fetch_child("service","section",$sid);
    $url = $this->link();
    foreach my $serv (@services) {
	my $upd_url = $this->link(seccmd=>"edserv",serv=>$serv->{service_id});
	my $del_url = $this->link(seccmd=>"delserv",serv=>$serv->{service_id});
	my $icon;
	if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$serv->{name}/icon.gif") {
	    $icon = "$config{server}{HTMLpath}/_Modules/$serv->{name}/icon.gif";
	}
	else {
	    $icon = "$config{server}{HTMLpath}/_Modules/Security/service.gif";
	}
	$out .= "<tr><td class=\"icon\"><img src=\"$icon\"></td><td>Website Service: <b>$serv->{name}</b>\n";
	if ($serv->{page_id}) {
	    my %page = $db->fetch("page",$serv->{page_id});
	    $out .= "<br>Running on page \"$page{title}\" ($page{filename})";
	}
	$out .= "<p><a href=\"$upd_url\">update</a> | <a href=\"$del_url\">delete</a>
</td></tr>\n";
    }
    # new service
    $url = $this->link(seccmd=>"newserv");
    $out .= "</table><a href=$url>Add New Website Service</a><br><br>\n";

    # add/remove users

    if (-e "$config{server}{CGIroot}/Modules/Member.pm") {
	$out .= "<table class=\"SecurityItem\" width=500 cellspacing=0 cellpadding=4>
<tr><th colspan=2>Website Members</th></tr>\n";
	my $nmembers = $db->count($config{auth}{user_table},{"section_id",$sid});
	$out .= "<tr><td class=\"icon\"><img src=\"$config{server}{HTMLpath}/_Modules/Member/icon.gif\"></td><td>This website has $nmembers member(s).<p>
<a href=\"$config{server}{CGIpath}/ctrl-panel.cgi/Member?site=$sid\">Manage Members</a></td></tr>\n</table><br><br>\n";
    }

    # mysite access level

    if (-e "$config{server}{CGIroot}/Modules/MySite.pm") {
	# MySite is installed
	my $url = $this->link(seccmd=>"mysite");
	my $site_type = $site{note};
	if ($site_type !~ /^(basic|regular|advanced)$/) { $site_type = "n/a"; }
	$out .= "<table class=\"SecurityItem\" width=500 cellspacing=0 cellpadding=4>
<tr><th colspan=2>MySite Status</th></tr>
<tr><td class=\"icon\"><img src=\"$config{server}{HTMLpath}/_Modules/MySite/icon.gif\"></td><td>MySite website type: <b>$site_type</b><p>
<a href=\"$url\">update</a></td></tr></table>\n";
	$out .= "<br><br>\n";
    }

    my $url = $this->link(seccmd=>undef,section_id=>undef);
    $out .= "<p>&lt;&lt; <a href=$url>Back to site listing</a>.</p>\n";

    return $out;
}

sub do_action {
    my $this = shift;
    my $cmd = $this->{input}{seccmd};
    my $db = $share{DB};
    my $sid = $this->{input}{section_id};
    $this->{site} = $db->fetch("section",$sid);
    my $out;
    
    # site info

    if ($cmd eq "mysite") {
	$out .= $this->mysite;
    }
    elsif ($cmd eq "edserv" || $cmd eq "newserv") {
	$out .= $this->serv_form;
    }
    elsif ($cmd eq "delserv") {
	$out .= $this->delserv;
    }
    elsif ($cmd eq "edkey") {
	$out .= $this->edkey;
    }
    elsif ($cmd eq "delkey") {
	$out .= $this->delkey;
    }
    elsif ($cmd eq "newkey") {
	$out .= $this->newkey;
    }
    elsif ($cmd eq "newadmin") {
	$out .= $this->newadmin;
    }
    elsif ($cmd eq "proadmin") {
	$out .= $this->proadmin;
    }
    elsif ($cmd eq "demuser") {
	$out .= $this->demuser;
    }
    elsif ($cmd eq "deluser") {
	$out .= $this->deluser;
    }
    elsif ($cmd eq "delsite") {
	$out .= $this->delsite;
    }
    else {
	$out .= "<p class=error>Unknown option.</p>";
    }
    return $out;
}

sub mysite {
    my $this = shift;
    my $db = $share{DB};
    my $sid = $this->{input}{section_id};
    my $site_type = $this->{site}{note};
    my $out;
    if ($this->{input}{mysite_type}) {
	$db->update("section",
		    {note=>$this->{input}{mysite_type}},
		    {section_id=>$sid});
	$site_type = $this->{input}{mysite_type};
	$out .= "<h2 class=help>Site type changed to <tt>$site_type</tt></h2>\n";
    }
    else {
	$site_type = $this->{site}{note};
    }
    my $url = $this->link();
    if ($site_type !~ /^(basic|regular|advanced)$/) { $site_type = "n/a"; }
    $out .= "
<h3>MySite Access Level</h3>
<p>This parameter controls the feature set that this website will have 
available, when accessed through the MySite content management interface.
Each level allows for different numbers of pages, uploaded images, and
use of dynamic web applications.  The options are:</p>
<ul>
<li><b>n/a</b> - MySite is not used with this website</li>
<li><b>basic</b> - the minimum level of functionality, which usually includes 
the ability to edit and publish a small number of pages.
<li><b>regular</b> - a medium level of functionality, which allows for a 
larger number of pages and images.
<li><b>advanced</b> - includes access to advanced site management tools,
such as dynamic web applications.
</ul>

<form method=post action=$url>
<p>MySite access is currently: 
<select name=mysite_type>\n";
    foreach my $type (qw(n/a basic regular advanced)) {
	$out .= "<option".
	    ($type eq $site_type ? " selected" : "").
	    ">$type</option>\n";
    }
    my $url = $this->link(seccmd=>undef);
    $out .= "</select> <input type=submit value=\"Update\"></p>
</form>
<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub delkey {
    my $this = shift;
    my $db = $share{DB};
    my $key = $this->{input}{key};
    my $adminkey = $db->fetch("member_site_link",$key);
    my $nkey = $db->count("member_site_link",{member_id=>$adminkey->{member_id}});	
    my $out;
    $db->trash_key("member_site_link",$key);
    if ($nkey == 1) {
	# this was the only key for this administrator
	my $admin = $db->fetch($config{auth}{user_table},$adminkey->{member_id});
	if ($admin) {
	    # found the administrator
	    my $site = $db->fetch("section",$admin->{section_id});
	    $out .= "<div class=WarningBox>\n";
	    if ($admin->{section_id}) {
		# the administrator is a regular member of a site
		$out .= "<p>WARNING: you removed the only 
administrator key for \"$admin->{name}\".  This user no longer has any
keys on this system.  However, they are a regular member of the 
website \"$site->{title}\", and have level $admin->{access} access to the 
ExSite administration tools.</p>\n";
		my $demoteurl = $this->link(seccmd=>"demuser",uid=>$admin->{member_id});
		$out .= "<p><a href=\"$demoteurl\">Demote this user to non-administrator status.</a></p>\n";
	    }
	    else {
		# administrator is now an orphan
		$out .= "<p>WARNING: you removed the only 
administrator key for \"$admin->{name}\".  This user no longer has any
keys on this system, and is not a member of any websites.  They still
have level $admin->{access} to the ExSite administration tools, however.</p>\n";
	    }
	    my $rurl = $this->link(seccmd=>undef,key=>undef);
	    $out .= "<p><a href=\"$config{server}{CGIpath}/webdb.cgi?action=edit&tab=member&id=$admin->{member_id}&reply=$rurl\">Edit this user.</a></p>\n";
	    my $durl = $this->link(seccmd=>"deluser",uid=>$admin->{member_id});
	    $out .= "<p><a href=\"$durl\">Delete this user completely.</a></p>\n";
	    $out .= "</div><br>\n";
	}
    }

    $out .= $this->show_site;
    return $out;
}

sub delserv {
    my $this = shift;
    my $db = $share{DB};
    my $serv = $this->{input}{serv};
    $db->trash_key("service",$serv);
    return $this->show_site;
}

sub deluser {
    my $this = shift;
    my $db = $share{DB};
    my $user = $this->{input}{uid};
    $db->trash_key($config{auth}{user_table},$user);
    return $this->show_site;
}

sub delsite {
    my $this = shift;
    my $db = $share{DB};
    my $sid = $this->{input}{section_id};
    return $this->error("No section ID!") if (! $sid);
    my @member = $db->fetch_m2m("member","section",$sid,"member_site_link");
    $db->trash_r("section",$sid);
    foreach my $m (@member) {
	my @key = $db->fetch_child("member_site_link","member",$m->{member_id});
	if (@key == 0) {
	    # no admin keys left
	    # this member is an orphan now - delete them
	    $db->trash_r("member",$m->{member_id});
	}
    }
    return $this->list;
}

# demote user

sub demuser {
    my $this = shift;
    my $db = $share{DB};
    my $user = $this->{input}{uid};
    $db->update($config{auth}{user_table},{access=>1},{member_id=>$user});
    return $this->show_site;
}

sub newkey {
    my $this = shift;
    my $db = $share{DB};
    my $key_id = $this->{input}{key};
    my $section_id = $this->{input}{section_id};
    if ($this->{input}{member_id}) {
	$db->insert("member_site_link",
		    {section_id=>$this->{input}{section_id},
		     member_id=>$this->{input}{member_id},
		     priveleges=>$this->{input}{priveleges}});
	return $this->show_site;
    }
    my ($serv,$page_id);
    my %key;
    if ($key_id) {
	%key = $db->fetch("member_site_link",$key_id);
    }
    my $url = $this->link();
    my $out = "<form method=post action=\"$url\">
<input type=hidden name=\"section_id\" value=\"$section_id\">
<h3>New Administrator Key</h3>
<p>Administrator keys grant special access to websites for managing 
content and web applications.</p>
<p>Select an administrator:<br>\n";
    $out .= $this->input_admin();
    $out .= "</p>
<p>Select the priveleges to be given to this administrator:<br>\n";
    $out .= $this->input_privs(undef);
    $out .= "</p>\n<p><input type=submit value=\" Submit \"></p>\n</form>";
    $url = $this->link(seccmd=>undef,serv=>undef);
    $out .= "<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub edkey {
    my $this = shift;
    my $db = $share{DB};
    my $key_id = $this->{input}{key};
    my $section_id = $this->{input}{section_id};
    if ($this->{input}{member_id}) {
	$db->update("member_site_link",
		    {section_id=>$this->{input}{section_id},
		     member_id=>$this->{input}{member_id},
		     priveleges=>$this->{input}{priveleges}},
		    {member_site_link_id=>$key_id});
	return $this->show_site;
    }
    my ($serv,$page_id);
    my %key;
    if ($key_id) {
	%key = $db->fetch("member_site_link",$key_id);
    }
    my $url = $this->link();
    my $out = "<form method=post action=\"$url\">
<input type=hidden name=\"section_id\" value=\"$section_id\">
<h3>Modify Administrator Key</h3>
<p>Select an administrator:<br>\n";
    $out .= $this->input_admin($key{member_id});
    $out .= "</p>
<p>Select the priveleges to be given to this administrator:<br>\n";
    $out .= $this->input_privs($key{priveleges});
    $out .= "</p>\n<p><input type=submit value=\" Submit \"></p>\n</form>";
    $url = $this->link(seccmd=>undef,serv=>undef);
    $out .= "<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub newadmin {
    my $this = shift;
    my $db = $share{DB};
    my $service_id = $this->{input}{serv};
    my $section_id = $this->{input}{section_id};
    if ($this->{input}{login}) {
	if ($this->{input}{passwd1} eq $this->{input}{passwd2}) {
	    my $id = $db->insert($config{auth}{user_table},
				 {name=>$this->{input}{name},
				  login=>$this->{input}{login},
				  password=>$this->{input}{passwd1},
				  email=>$this->{input}{email},
				  access=>$this->{input}{access},
				  section_id=>$this->{input}{section_id},
			      });
	    $db->insert("member_site_link",
			{member_id=>$id,
			 priveleges=>$this->{input}{priveleges},
			 section_id=>$section_id} );
	    return $this->show_site;
	}
	else {
	    return "<p class=error>The passwords you entered do not match.</p>
<p><a href=\"javascript:history.back()\"><< back</a></p>\n";
	}
    }
    my $url = $this->link();
    my $out = "<form method=post action=\"$url\">
<input type=hidden name=section_id value=$section_id>
<h3>New Administrator</h3>
<p>Full Name:<br>
<input type=text size=40 name=name></p>
<p>Email:<br>
<input type=text size=40 name=email></p>
<p>Login:<br>
<input type=text size=20 name=login></p>
<p>Password (enter twice to confirm):<br>
1. <input type=password size=20 name=passwd1><br>
2. <input type=password size=20 name=passwd2></p>
<p>Administrator type:<br>
<input type=radio name=access value=2> website adminstrator (limited access)<br>
<input type=radio name=access value=3> system adminstrator (unlimited access)</p>
<p>Select the priveleges to be given to this administrator:<br>\n";
    $out .= $this->input_privs(undef);
    $out .= "</p><p><input type=submit value=\"Submit\"></p>
<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub proadmin {
    my $this = shift;
    my $db = $share{DB};
    my $service_id = $this->{input}{serv};
    my $section_id = $this->{input}{section_id};
    if ($this->{input}{access} && $this->{input}{member_id} && $this->{input}{priveleges}) {
	$db->update($config{auth}{user_table},
		    {access=>$this->{input}{access}},
		    {member_id=>$this->{input}{member_id}} );
	$db->insert("member_site_link",
		    {member_id=>$this->{input}{member_id},
		     section_id=>$section_id,
		     priveleges=>$this->{input}{priveleges}} );
	return $this->show_site;
    }
    elsif ($this->{input}{level} || $this->{input}{member_id} || $this->{input}{priveleges}) {
	return "<p class=error>You did not provide sufficient information.</p>
<p><a href=\"javascript:history.back()\"><< back</a></p>\n";
    }
    my $url = $this->link();
    my $out = "<form method=post action=\"$url\">
<h3>Promote User to Administrator Status</h3>\n";
    my @user = $db->fetch_all($config{auth}{user_table},$config{auth}{user_sort_column});
    $out .= "<select name=member_id>\n<option value=\"0\">== select user ==</option>\n";
    foreach my $admin (@user) {
	next if ($admin->{access} > 1);
	$out .= "<option value=\"$admin->{member_id}\">$admin->{name} (login: $admin->{login})</option>\n";
    }
    $out .= "</select></p>
<p>Promote this user to:<br>
<input type=radio name=access value=2> website adminstrator (limited access)<br>
<input type=radio name=access value=3> system adminstrator (unlimited access)</p>
<p>Select the priveleges to be given to this administrator:<br>\n";
    $out .= $this->input_privs(undef);
    $out .= "</p><p><input type=submit value=\"Submit\"></p>
<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub serv_form {
    my $this = shift;
    my $db = $share{DB};
    my $service_id = $this->{input}{serv};
    my $section_id = $this->{input}{section_id};
    if ($this->{input}{name}) {
	if ($service_id) {
	    $db->update("service",
			{name=>$this->{input}{name},
			 page_id=>$this->{input}{page_id}},
			{service_id=>$service_id});

	}
	else {
	    $db->insert("service",
			{section_id=>$section_id,
			 name=>$this->{input}{name},
			 page_id=>$this->{input}{page_id}});
	}
	return $this->show_site;
    }
    my $url = $this->link();
    my $out = "<form method=post action=\"$url\">
<h3>Configure Web Application Licenses/Services</h3>
<ul>
<li> To grant a license to use a web application to this website, select
the web application from the first selector, below.</li>
<li> To direct all links from a particular web application to a particular
page that should service that application, select web application in the
first selector (this also grants a license), and the target page in the
second selector.</li>
</ul>
<ol>
<li><p>Select the web application to work with:<br>";
    my ($name,$page_id);
    if ($service_id) {
	my %serv = $db->fetch('service',$service_id);
	$name = $serv{name};
	$page_id = $serv{page_id};
    }
    $out .= $this->input_dcd($name);
    $out .="</p></li>
<li><p>Some web applications generate links back to themselves.  If you would 
like these links to be serviced by a particular page, select that page here.  Otherwise
the links will go back to the originating page.<br>\n";
    $out .= $this->input_page($section_id,$page_id);
    my $url = $this->link(seccmd=>undef);
    $out .= "</p></li></ol>\n<p><input type=submit value=\"Submit\"></p>
<p>&lt;&lt; <a href=\"$url\">Back</p>\n";
    return $out;
}

sub input_dcd {
    my ($this,$name) = @_;
    my $c = new ExSite::ContentBase;
    my @dcd = $c->get_dcd_all;
    
    my $out = "<select name=name>\n<option>== select application ==</option>\n";
    foreach my $dcd (@dcd) {
	$out .= "<option".
	    ($dcd eq $name ? " selected" : "").
	    ">$dcd</option>\n";
    }
    $out .= "</select>\n";

    return $out;
}

sub input_page {
    my ($this,$sid,$pid) = @_;
    my $db = $share{DB};
    my @pages = $db->fetch_match("page",{section_id=>$sid,type=>"page"},"title");
    my $out = "<select name=page_id>\n<option value=\"0\">== select page ==</option>\n";
    foreach my $p (@pages) {
	$out .= "<option value=\"$p->{page_id}\"".
	    ($p->{page_id} eq $pid ? " selected" : "").
	    ">$p->{title} ($p->{filename})</option>\n";
    }
    $out .= "</select>\n";

    return $out;
}

sub input_admin {
    my ($this,$member_id) = @_;
    my $db = $share{DB};
    my @user = $db->fetch_all($config{auth}{user_table},$config{auth}{user_sort_column});
    my $out = "<select name=member_id>\n<option value=\"0\">== select administrator ==</option>\n";
    foreach my $admin (@user) {
	next if ($admin->{access} < 2);
	$out .= "<option value=\"$admin->{member_id}\"".
	    ($admin->{member_id} eq $member_id ? " selected" : "").
	    ">$admin->{name} (level: $admin->{access}; login: $admin->{login})</option>\n";
    }
    $out .= "</select>\n";
    if (! $member_id) {
	my $newuserurl = $this->link(seccmd=>"newadmin");
	my $promoteurl = $this->link(seccmd=>"proadmin");
	$out .= "<br><br>
If your preferred administrator does not appear in the above list of 
admininstrators, then you must either  
<a href=\"$newuserurl\">create a new administrator</a>
or <a href=\"$promoteurl\">promote a regular user</a> to administrator level.\n";
    }
    return $out;
}

sub input_privs {
    my ($this,$privs) = @_;
    my $db = $share{DB};
    my $design_checked = ($privs =~ /designer/) ? " checked" : "";
    my $editor_checked = ($privs =~ /editor/) ? " checked" : "";
    my $admin_checked =  ($privs =~ /admin/) ? " checked" : "";
    my $out = "<input type=\"checkbox\" name=\"priveleges\" value=\"editor\"$editor_checked>
Editor - can update editorial content and publish pages.<br>
<input type=\"checkbox\" name=\"priveleges\" value=\"designer\"$design_checked>
Designer - can update design files and templates.<br>
<input type=\"checkbox\" name=\"priveleges\" value=\"administrator\"$admin_checked>
Administrator - can add and remove pages.\n";
    return $out;
}

1;
