package Modules::WebDB;

#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

=pod

WebDB : multi-purpose web-based database management

=cut

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Form;
use ExSite::Input;

# inherit from Base class
use Modules::BaseDCD;
use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub new {
    my $this = shift;
    my $obj = {};
    my $class = ref($this) || $this;
    bless $obj, $class;

    # change behaviour of Form and Report modules to exsite mode
    $config{form}->{mode} = $config{report}->{linkmode} = "exsite";
    ###### warning, this change will be persistent in mod_perl!

    $obj->read;  # inherits BaseDCD::read()
    return $obj;
}

sub write {
    my ($this,$options) = @_;
    my %opt = &DecodeString($options);
    my $action = $this->{input}{action} || $opt{action};
    return "" if ($action eq "null");  # dummy call
    my $table = $this->{input}{tab} || $opt{tab};

    if (! exists $this->{DB}) {
	# We need a database handle, but the default handle might not be a 
	# Form object...  so we'll create a new Form, just to make sure.
	# (FIXME: DB should sense whether the currently open database is 
	# the correct one, and if so, use it, to avoid opening a redundant
	# connection.)

#	if ($opt{db}) {
#	    # a special database was selected
#	    $this->{DB} = new ExSite::Form(%{$config{server}->{$opt{db}}},
#					   save=>0);
#	    # (don't save this DB handle, or it will override the one we
#	    # use for the CMS)
#	}
#	else {
#	    # the default database was selected
#	    $this->{DB} = new ExSite::Form;
#	}

	$this->{DB} = $share{DB};
 
        # WARNING: we disconnect at bottom, so changes to this logic should
        # also be reflected there.
    }

    # if the link flag is set, we publish a link to the function, 
    # not the function itself.

    my $link_flag;
    my $out;
    if ($table) {
	my $img = "$config{server}{HTMLpath}/_Modules/WebDB";
	my $top = &relink(tab=>undef,id=>undef,pid=>undef,ptab=>undef,action=>undef);
	$out = "<p><a href=$top><IMG SRC=$img/$config{db_ops}{list}{img} HEIGHT=$config{db_ops}{list}{img_height} WIDTH=$config{db_ops}{list}{img_width} ALIGN=middle border=0> Top (all tables)</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n";
	if ($this->{input}{id}) {
	    my $url = &relink(id=>undef,pid=>undef,ptab=>undef,action=>undef);
	    my $tname = $this->{DB}{map}->table_title($table,1);
	    $out .= "<a href=$url><IMG SRC=$img/$config{db_ops}{list}{img} HEIGHT=$config{db_ops}{list}{img_height} WIDTH=$config{db_ops}{list}{img_width} ALIGN=middle border=0> All $tname</a>\n";
	}
    }
    if ($action =~ /^link-(.+)$/o) {
	$out .= $this->link($1);
    }
    elsif ($action =~ /edit/) {     # display a form to edit a record
	$out .= $this->form;
    }
    elsif ($action =~ /new/) {      # display a form to add a record
	$out .= $this->form;
    }
    elsif ($action =~ /query/) {   # search for records
	$out .= $this->search(\%opt);
    }
    elsif ($action =~ /copy/) {     # display a form to copy a record
	$out .= $this->copy(\%opt);
    }
    elsif ($action =~ /delete/) {   # delete a record
	$out .= $this->rm(\%opt);
    }
    elsif ($action =~ /insert|update|search/) {   # process a form
	$out .= $this->doform(\%opt);
    }
    elsif ($action =~ /report/) {   # interactive report
	$out .= $this->report(\%opt);
    }
    else {
	$out .= $this->dbadm(\%opt);
    }
    return $out;
}

# ioctl : all operations that don't qualify as reads or writes
#         invoked as <!--BaseDCD.ioctl(request)-->

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Database Manager";
    }
    elsif (/ControlPanel/) {
	#return \&write;
	return "$config{server}{CGIpath}/webdb.cgi";
    }
    elsif (/isRestricted/) {
	return exists $config{WebDB}{isRestricted} ? 
	    $config{WebDB}{isRestricted} : 0;
    }
    elsif (/ModuleInfo/) {
	return "The Database Manager provides functions for searching, 
browsing, editing, and adding records to any of the tables in
your website's database.  It is an expert-level tool that gives you
the ability to \"hack\" your website's data.";
    }
}

#-----------------------------------------------------------------------
# dbadm() - general-purpose db admin tool
#
# This function accepts the following option keys/values:
#
# tab=xxx   the table to manage;
# id=nnn    the key of a particular record in the above table;
#           - without this parameter, members of the table are listed
#           for selection.
#           - with this parameter, the associated members of the
#           subtables are listed, with options to edit, delete, or add
#           to the table.
# stab=xxx  the subtable(s) to manage;
#           selects which subtables to list;  by default all are shown.
# ptab=xxx  the parent table  } must both be specified
# pid=nnn   the parent id     } to have a valid meaning
#           - selects which members of the table are listed;  by default
#           all members belonging to the account are listed.
#-----------------------------------------------------------------------

sub dbadm {
    my ($this,$opt) = @_;
    my $dbargs = $this->{input}{tab} ? $this->{input} : $opt;
    $dbargs->{title} = 1;   # title all reports
    $dbargs->{all} = 1;     # allow all valid db ops
    my $out .= "<table align=right><tr><td>".$this->quickhelp."</td></tr></table>";
    $out .= $this->{DB}->report_relational($dbargs);
    my $img = "$config{server}->{HTMLpath}$config{server}->{imgdir}";
    return $out;
}

#-----------------------------------------------------------------------
# doform : generic form processing tool
#-----------------------------------------------------------------------

sub doform {
    my $this = shift;
    return $this->{DB}->do;
}

#-----------------------------------------------------------------------
# form : generic form generation tool
#
# query arguments:
# tab=table
# id=row to edit
# ptab,pid=foreign key to set on inserts
# [action=search]
# [reply=reply_page_url]
#
# hide=list of rows to not display
# show=list of rows to display
#-----------------------------------------------------------------------

sub form {
    my ($this,$opt) = @_;
    my (%query,$title,%data,%extra,@hide,@show,$db,$approved,$out);

    %query = $this->{input}{tab} ? %{$this->{input}} : %$opt;
    $db = $this->{DB};

    # any rows to hide/show?
    if (defined $query{hide}) {
	@hide = split(/,/,$query{hide});
    }
    if (defined $query{show}) {
	@show = split(/,/,$query{show});
    }

    my $access = $db->authorize;

    # set the reply page
    if (defined $query{reply}) {
	$extra{reply} = $query{reply};
	delete $query{reply};
    }
    elsif (defined $ENV{HTTP_REFERER}) {
	###################################### DANGER !?
	if ($ENV{HTTP_REFERER} !~ /form\.cgi/) {
	    # DO NOT RETURN TO THE FORM SCRIPT - you might re-submit an update
	    $extra{reply} = $ENV{HTTP_REFERER};
	}
    }

    $title = $db->title($query{tab},$query{id});

    if (defined $query{id}) {
	# this is an edit request
	$approved = $db->approve("update",$query{tab},$query{id});
	$out .= "<h2>Edit $title</h2>\n";
	#%data = $db->fetch($query{tab},$query{id});
	if ($approved) {
	    if ($query{id}) {
		$out .= $db->make(table=>$query{tab},
				  record=>$query{id},
				  hide=>\@hide,
				  show=>\@show,
				  extra=>\%extra,
				  group=>$query{group});
	    }
	    else {
		$out .= "No such $query{tab}.<P>\n";
	    }
	}
	else {
	    $out .= "<P CLASS=error>You do not have permission for this operation.</P>\n";
	}
    }
    else {
	# no id - must be an insert
	$approved = $db->approve("insert",$query{tab});
	# some values may be provided through query
	%data = %query;
	if (defined $query{ptab} && defined $query{pid}) {
	    my $refcol =
		$db->{map}->get_reference_column($query{tab},$query{ptab});
	    $data{$refcol} = $query{pid};
	    $out .= "<h2>Add new $title</h2>\n";
	    if ($approved) {
		$out .= $db->make(table=>$query{tab},
				  data=>\%data,
				  hide=>\@hide,
				  show=>\@show,
				  extra=>\%extra,
				  group=>$query{group});
	    }
	    else {
		$out .= "<P CLASS=error>You do not have permission for this operation.</P>\n";
	    }
	}
	else {
	    # hmm, no relational references were passed
	    my @ptab = $db->{map}->get_parents();
	    if (@ptab == 0) {
		# okay, this table has no parents
		$out .= "<h2>Add new $title</h2>\n";
		if ($approved) {
		    $out .= $db->make(table=>$query{tab},
				      data=>\%data,
				      hide=>\@hide,
				      show=>\@show,
				      extra=>\%extra,
				      group=>$query{group});
		}
		else {
		    $out .= "<P CLASS=error>You do not have permission for this operation.</P>\n";
		}
	    }
	    elsif ($access == $config{auth}->{root_auth_level}) {
		# superadmin mode can force inserts of unparented rows
		$out .= "<h1>Add new $title</<h1>\n";
		$out .= $db->make(table=>$query{tab},
				  data=>\%data,
				  hide=>\@hide,
				  show=>\@show,
				  extra=>\%extra,
				  group=>$query{group});
	    }
	    else {
		# regular users cannot insert unreferenced data
		$out .= "<P CLASS=error>You do not have permission for this operation.</P>\n";
	    }
	}
    }
    return $out;
}

#-----------------------------------------------------------------------
# search : generic database search tool
#
# With no query string, generates a generic search form in which the
# user may enter keywords to search for.  A generic search looks for
# instances of these keywords in any columns with a search attribute
# of "general".
#
# Alternatively, a more specific (and more efficient) search can be
# conducted on a single table, by providing the following values in
# the query string:
#
# tab=table
# hide=optional list of rows to not display
# show=optional list of rows to display
#-----------------------------------------------------------------------

sub search {
    my ($this,$opt) = @_;
    # local symbols
    my (%query, $title, %data, %extra, @hide, @show, $db, $out);
    $db = $this->{DB};
    %query = scalar keys %{$this->{input}} ? 
	%{$this->{input}} : %$opt;

    # any rows to hide/show?
    if (defined $query{hide}) {
	@hide = split(/,/,$query{hide});
    }
    if (defined $query{show}) {
	@show = split(/,/,$query{show});
    }

    #$out = &ShowHash(%query);

    $title = $db->title($query{tab});
    if (defined $query{tab}) {
	# specific search
	$out .= "<h2>$title Search</h2>\n";
	$out .= "<P>Enter values only into those fields you wish to search on.
Any records containing the text string you entered (exactly as you
entered it) will be found.</P>\n";
	$out .= $db->make(table=>$query{tab},hide=>\@hide,show=>\@show,search=>1);
    }
    elsif ($query{action} eq "search_all") {
	$out .= "<H3>Search Results:</H3>\n";

	# search string
	my $str = "%".$query{search}."%";

	# get access level to database
	my $level = $db->authorize();

	# get list of searchable tables
	my $tablemap = $db->{map}->get_tablemap();
	my @table = ();
	my %found;
	my $total_found = 0;
	foreach my $table (keys %$tablemap) {
	    if ($tablemap->{$table}->{search} <= $level) {
		# build a SQL select statement for the table
		my @where = ();
		my $map = $db->{map}->get_map($table);
		for (my $icol = 0; $icol < $db->{map}->get_ncol($table); $icol++) {
		    if ($map->{search}->[$icol] eq "general") {
			push @where,"$map->{column}->[$icol] LIKE \"$str\"";
		    }
		}
		# fetch matching rows
		if (@where > 0) {
		    my $sql = "SELECT * FROM $table WHERE (".join(" || ",@where).")";
		    my $found = $db->fetch_data_custom($sql);
		    if (@$found > 0) {
			# report the found items
			$out .= "<H3>".$db->{map}->table_title($table,1).":</H3>\n";
			$out .= $db->report($table,$found,(link=>1, edit=>1, sort=>1));
			$total_found += @$found;
		    }
		}
	    }
	}
	if ($total_found == 0) {
	    $out .= "<P CLASS=\"error\">Nothing relevant was found.</P>\n";
	}
	$out .= "<HR>\n";
	$out .= $this->general_search_form;
    }
    else {
	# no input - prompt for a general search
	$out .= $this->general_search_form;
    }
    return $out;
}

sub general_search_form {
    my $out;
    $out .= "<H2>Search</H2>
<P><FORM METHOD=GET ACTION=$config{server}->{CGIpath}/search.cgi>
<INPUT TYPE=HIDDEN NAME=action VALUE=\"search_all\">
<INPUT TYPE=TEXT NAME=search>
<INPUT TYPE=SUBMIT VALUE=\"Search\">
</FORM></P>\n";
}

sub copy {
    my ($this,$opt) = @_;
    # local symbols
    my (%query, $title, %data, %extra, @hide, @show, $db, $out);
    $db = $this->{DB};
    %query = scalar keys %{$this->{input}} ? 
	%{$this->{input}} : %$opt;

    my $out;
    if ($query{tab} && $query{id}) {
	my %data;
	if ($db->approve("insert",$query{tab})) {
	    %data = $db->fetch($query{tab},$query{id});
	    # remove any serial primary key
	    my $key = $db->get_key($query{tab});
	    my $keytype = $db->{map}->get($query{tab},$key,"datatype");
	    if ($keytype = "key#") {
		delete $data{$key};
	    }
	    $out = "<H3>Copy ",$db->title($query{tab},$query{id}),"</H3>\n";
	    $out .= "<P CLASS=\"help\"><b>NOTE ON COPYING:</b> The record will not be
copied until this form is submitted.  Only those fields that you have
permission to read will be copied.  The copy operation also copies references
to related tables and records, so the new record will have the same data
relations as the original.  You may modify any of the copied data before
submitting.  Selecting \"Reset\" will restore the copied record's data.
Selecting \"Cancel\" will cancel the copy.</P>\n";
	    $out .= $db->make(table=>$query{tab},data=>\%data,extra=>\%extra);
	}
	else {
	    $out .= "<SPAN CLASS=\"error\">\n";
	    $out .= "<H3>Problem:</H3>\n<P>You do not have permission to perform this operation.</P>\n";
	    $out .= "</SPAN>\n";
	}
    }
    else {
	$out .= "<SPAN CLASS=\"error\">\n";
	$out .= "<H3>Problem:</H3>\n<P>No table/record specified.</P>\n";
	$out .= "</SPAN>\n";
    }
    return $out;
}


sub rm {
    my ($this,$opt) = @_;
    my %query = scalar keys %{$this->{input}} ? 
	%{$this->{input}} : %$opt;
    my $db = $share{DB};
    my $out;
 
    # connect to database
    if ($query{tab} && $query{id}) {
	if ($db->approve("delete",$query{tab},$query{id})) {
	    $db->trash_r($query{tab},$query{id},1);
	    my $p = $share{Page} || $share{PageBuf};
	    my $reply = $query{reply} || $ENV{HTTP_REFERER};
	    if ($p) { 
		$p->location($reply); 
		return;
	    } 
	    else {
		my $title = $db->title($query{tab},$query{id});
		$out .= "<h3>$title has been moved to the trashbin</h3>
<p><a href=\"$reply\">Continue</a></p>\n";
	    }
	}
	else {
	    $out .= "<SPAN CLASS=\"error\">\n";
	    $out .= "<H3>Problem:</H3>\n<P>You do not have permission to perform this operation.</P>\n";
	    $out .= "</SPAN>\n";
	}
    }
    else {
	$out .= "<SPAN CLASS=\"error\">\n";
	$out .= "<H3>Problem:</H3>\n<P>No table/record specified.</P>\n";
	$out .= "</SPAN>\n";
    }
    return $out;
}

# report : paginated, sortable reports

sub report {
    my ($this,$opt) = @_;
    my %arg = scalar keys %{$this->{input}} ? %{$this->{input}} : %$opt;
    my $p = $share{Page};
    my $db = $share{DB};
    my $out;

    my %query;               # query spec built from %arg
    my %options;             # arguments passed to &report
    my @tab;

    if (exists $arg{query}) {
	# query is complete and encoded
	$options{query} = $db->decode_query($arg{query});
	@tab = @{$options{query}->{table}};
    }
    else {
	# query is decoded
	my $table = $arg{table} ? $arg{table} : $arg{tab};
	@tab = split /,/, $table;
	$query{table} = \@tab;
	delete $arg{table};
	delete $arg{tab};
	if ($arg{columns}) {
	    my @col = split /,/, $arg{columns};
	    $query{columns} = \@col;
	    delete $arg{columns};
	}
	if ($arg{start}) {
	    my $start = $arg{start};
	    delete $arg{start};
	}
	if ($arg{num}) {
	    my $number = $arg{num};
	    delete $arg{num};
	}
	if ($arg{type}) {
	    $options{type} = $arg{type};
	    delete $arg{type};
	}
	# everything left in args will be used as a match hash
	if (scalar keys %arg > 0) {
	    $query{match} = \%arg;
	}
	$options{query} = \%query;	
    }
    # control options are not part of query, and may be passed separately    
    if (exists $arg{link}) {
	$options{link} = $arg{link};
	delete $arg{link};
    }
    if (exists $arg{edit}) {
	$options{edit} = $arg{edit};
	delete $arg{edit};
    }
    if (exists $arg{copy}) {
	$options{copy} = $arg{copy};
	delete $arg{copy};
    }
    if (exists $arg{delete}) {
	$options{delete} = $arg{delete};
	delete $arg{delete};
    }
    if (exists $arg{all_edit}) {
	$options{all_edit} = $arg{all_edit};
	delete $arg{all_edit};
    }

    # check to make sure we haven't deleted all keys in our match hash
    if (exists $query{match}) {
	if (keys %{$query{match}} == 0) {
	    delete $query{match};
	}
    }

    if (@tab == 0) {
	$out .= "<P CLASS=\"error\">Bad report specification: no tables to report on.</P>\n";
    }
    elsif (@tab == 1) {
	# simple report
	$options{title} = $db->{map}->table_title($tab[0],1);
	$out .= $db->report(\%options);
    }
    else {
	# multi-table report
	### NB: same match hash is used for each table;  this will cause SQL errors if the
	### match columns don't exist in all tables!
	foreach my $table (@tab) {
	    $options{title} = $db->{map}->table_title($table,1);
	    $options{table} = [ $table ];
	    $out .= $db->report(\%options);
	}
    }
    return $out;
}

# link : link back to the same page, with modified query string

sub link {
    my ($this,$action) = @_;
    my %query = %{$this->{input}};
    $query{action} = $action;
    #my $url = "$config{server}->{CGIpath}/page.cgi";
    if (! $query{_id}) {
	$query{_id} = $share{Page}->{page_id};
    }
    my $q = &EncodeHash(%query);
    return $this->{DB}->link($action,$q,"page.cgi");
}

# add quick-help guide

sub quickhelp {
    my $this = shift;
    my $img = "$config{server}->{HTMLpath}/_Modules/WebDB";
    my %report = (
		  title=>"Option Key",
		  data=>[
			 [ "<IMG SRC=$img/$config{db_ops}{list}{img} HEIGHT=$config{db_ops}{list}{img_height} WIDTH=$config{db_ops}{list}{img_width} ALIGN=middle>", "<b>list</b> records" ],
			 ["<IMG SRC=$img/$config{db_ops}{view}{img} HEIGHT=$config{db_ops}{view}{img_height} WIDTH=$config{db_ops}{view}{img_width} ALIGN=middle>", "<b>view</b> record info" ],
			 ["<IMG SRC=$img/$config{db_ops}{edit}{img} HEIGHT=$config{db_ops}{edit}{img_height} WIDTH=$config{db_ops}{edit}{img_width} ALIGN=middle>", "<b>edit</b> this item" ],
			 ["<IMG SRC=$img/$config{db_ops}{copy}{img} HEIGHT=$config{db_ops}{copy}{img_height} WIDTH=$config{db_ops}{copy}{img_width} ALIGN=middle>", "<b>copy</b> this item" ],
			 ["<IMG SRC=$img/$config{db_ops}{delete}{img} HEIGHT=$config{db_ops}{delete}{img_height} WIDTH=$config{db_ops}{delete}{img_width} ALIGN=middle>", "<b>delete</b> this item" ],
			 ["<IMG SRC=$img/$config{db_ops}{searchquery}{img} HEIGHT=$config{db_ops}{searchquery}{img_height} WIDTH=$config{db_ops}{searchquery}{img_width} ALIGN=middle>", "<b>search</b> for items of this type" ],
			 ["<IMG SRC=$img/$config{db_ops}{new}{img} HEIGHT=$config{db_ops}{new}{img_height} WIDTH=$config{db_ops}{new}{img_width} ALIGN=middle>", "<b>add</b> a new item of this type" ],
			 ],
	      );
    my $r = new ExSite::ReportBuilder(%report);
    return $r->make;
}

sub DESTROY {
    my $this = shift;
    # clean up our connections
    #$this->{DB}->{dbh}->disconnect;   ########???
    # chnage Form and Report behaviour back to CGI mode
    $config{form}->{mode} = $config{report}->{linkmode} = "cgi";
}

### end

1;
