#!/usr/bin/perl
#-----------------------------------------------------------------------
# ctrl-panel.cgi - generate a control panel for a web app
#
# Usage:  /cgi-bin/ctrl-panel.cgi/WEBAPP?options
#
# WEBAPP is the name of an application installed in the Modules area.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::PageBuf;
use ExSite::Form;  # use Form if inputting or processing forms
use ExSite::Misc;

use vars qw($db $page %query);

&exsite_init;
$db = new ExSite::Form;
%query = &DecodeQuery;

# enable page buffering for better redirect control

my @path = split /\//, $ENV{PATH_INFO};
shift @path;
my $module = shift @path;

$page = new ExSite::PageBuf(title=>"$module - $config{site}{name}",
			    headtype=>1);

my ($icon,$name,$info,$out);

if ($db->authorize) {
    my $stat = eval "require Modules::$module";
    my $app;
    if ($stat) {
	$app = eval "new Modules::$module";
	# look for a control panel
	if ($app) {
	    my $allow = (! $app->ioctl('isRestricted')) || 
		$db->license_control_panel($module);
	    if ($allow) {
		$icon = $app->icon;
		$name = $app->ioctl('ModuleName') || $module;
		$info = $app->ioctl('ModuleInfo');
		my $panel = $app->ioctl('ControlPanel');
		if (ref $panel eq "CODE") {
		    $out .= ( $app->$panel || $@ );
		}
		else {
		    $out .= "<h4>Error executing $module control panel</h4>\n";
		}
	    }
	    else {
		$out .= "<h4>$module: Permission denied (no license)</h4>\n";
	    }
	}
	else {
	    $out .= "<h4>No control panel found for $module</h4>\n";
	}
    }
    else {
	$out .= "<h4>Failed to start module $module</h4>\n<p>$@</p>\n";
    }
}
else {
    $out .= "<h4>Permission denied.</h4>\n";
}

my $help;
if (-e "$config{server}{HTMLroot}$config{server}{HTMLpath}/_Modules/$module/help.html") {
    $help = "<a href=\"javascript:popup_help('$config{server}{CGIpath}/help.cgi/_Modules/$module/help.html')\">help</a>";
}

my $header = "<table width=\"100%\" cellspacing=0 cellpadding=0 border=0>
<tr><td rowspan=2><img src=\"$icon\" align=\"left\"></td>
<td class=titlebar_noborder><span class=\"title\">$name</span><br>\n$info</td>
</tr>
<tr><td valign=bottom align=right class=titlebar_noborder>
$help &nbsp;&nbsp;
<a href=\"$config{server}{CGIpath}/home.cgi\">close</a> 
</td></tr></table>\n";

$page->print("<div align=\"center\">
<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\" width=\"90%\" class=\"framed\">
      <tbody><tr>
        <td class=\"titlebar\">
$header
        </td>
      </tr>
      <tr>
        <td bgcolor=\"white\"><div class=\"htmlbox\">

$out

        </div></td>
      </tr>
    </tbody></table>
</div>
<br>\n");
$page->show;

### END

