#!/usr/bin/perl
#-----------------------------------------------------------------------
# dcd.cgi - call DCD directly
#
# DCDs (web applications, or plug-ins) are normally invoked through 
# special CMS tags that are processed using page.cgi.  In some cases
# we would like to call the DCDs directly, however;  dcd.cgi can be 
# used in this case.  This is normally used internally by ExSite --
# it would not normally be invoked directly from the browser.
#
# USAGE:
#    dcd.cgi/Module?dcd_query_data&_ajax=ID&_ref=referrer_url
#
# The referrer url is used to determine page and site position, in cases
# where the DCD needs a context to produce results.
#
# _ajax=ID directs the DCD to relink back into the DCD by using AJAX 
# (httpRequest) methods to fetch new DCD content using dcd.cgi, and
# inserting the fetched content into DOM element "ID".
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
# TODO:
# - allow for content expansion of the output
# - link inter-page services correctly
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Page;
use ExSite::Form;
use ExSite::Input;

use vars qw($db $dcdname $page_id $stat $dcd $license_ok 
	    $script $input $output $ref $query $prevquery);

&exsite_init;

print "Content-type: text/html\n\n";

my (undef,$dcdname) = split(/\//,$ENV{PATH_INFO});

if ($dcdname) { 
    my $in = new ExSite::Input;
    my $q = $in->query;
    # httpRequest tools do not set the HTTP_REFERER, so we have to 
    # manually insert referer data into our URL
    my $ref = $q->{_ref};
    $ref =~ /\/([^\/]+\.cgi)\?(.*_id=(\d+).*)/;
    $script = $1;
    $prevquery = $2;
    $page_id = $3;
    # should we use AJAX for re-links, too?
    if ($q->{_ajax}) {
	$share{ajax_target} = $q->{_ajax};
    }
    $db = new ExSite::Form;
    $stat = eval "require Modules::$dcdname";
    if ($stat) {
	if ($dcd = eval "new Modules::$dcdname") { 
	    # check licensing
	    $license_ok = 1;
	    if ($dcd->ioctl("isRestricted")) {
		# this service requires licenses
		if ($page_id) {
		    my $p = $db->fetch("page",$page_id);
		    my @licenses = $share{DB}->fetch_match("service",
							   {section_id => $p->{section_id},
							    name => $dcdname }
							   );
		    $license_ok = (@licenses > 0);
		}
		if (! $license_ok) {
		    # log the problem
		    $dcd->warn("$dcdname: unlicensed access from dcd.cgi");
		}
	    }
	    
	    if ($license_ok) {

		my $options;
		if (! $q->{_ajax}) {
		    # reset the environment to make it look like we came 
		    # from page.cgi
		    $ENV{SCRIPT_NAME} =~ s/dcd.cgi$/$script/;
		    $ENV{SCRIPT_FILENAME} =~ s/dcd.cgi$/$script/;
		    $options = $ENV{QUERY_STRING};
		    $ENV{QUERY_STRING} = $prevquery;
		    delete $ENV{PATH_INFO};
		    delete $share{input};
		}

		# create a page object, so inter-page services know how
		# to link us to service pages
		$share{Page} = new ExSite::Page(id=>$page_id);
		$share{Page}->set_context("parent");

		# execute the DCD method
		if ($q->{_ajax}) {
		    $output = $dcd->write() || $@;
		}
		else {
		    $output = $dcd->write($options) || $@;
		}
	    }
	    else {
		$output = "$dcdname: permission denied (no license)";
	    }
	}
	else {
	    $output = "$dcdname: $@";
	}
    }
    else {
	$output = "$dcdname: $@";
    }
}

print $output;

