#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_dyncontent.cgi: custom dialog used by HTML editor for adding 
# dynamic content
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Content;
use ExSite::Input;
use myConfig; 

use vars qw(%data $uid $access $OUT $db $title $file $in $input_data);

&exsite_init;
$config{page_buffering} = 0;
$in = new ExSite::Input;
$input_data = $in->combine();
%data = %$input_data;
$db = new ExSite::DB;
$access = $db->authorize;
$title = "Insert Web Application";
$file = "dlg_dyncontent.cgi";

# access denied
if ($access==0){
	print "Location: login.cgi\n\n";
# access denied
}elsif ($access < 2){
	$OUT .= &PrintHeader;
	$OUT .= "<p>Access denied.  Administrators only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
# get icon url
}elsif ($data{action} eq "geturl"){
	print &getUrl($data{name});
# get module parameters
}elsif ($data{action} eq "getparam"){
	print &getParam($data{name},$data{section_id});
}else{
	$OUT .= &PrintHeader;	
	$OUT .= &PrintForm;
	$OUT .= &PrintFooter;
	print $OUT;
}
#---------------------------------------------------------------------------
#
# Running DynContentParameter routine to create 
# the optional parameter input field for the dialog
#
# - the name of the input field has to be 'param'.
#
sub getParam{
	my ($name,$sectionid) = @_;
	my ($out,$test,$myobj,$parameter,$defaultInput);
	$out = "content-type: text/html\n\n";
	$defaultInput = "<table><tr><td><b>Parameter (optional):</b></td><td><input type=\"text\" name=\"param\" value=\"\"></td></table>";
	$test = eval("require Modules::$name");
	# return a default input if a module doesn't exist
	return $out.$defaultInput if !$test;
	$myobj = eval("new Modules::$name");
	# run DynContentParameter routine
	$parameter = $myobj->ioctl("DynContentParameter",$sectionid);	
	if ($parameter){
		$out .= $parameter;
	}else{
		# return a default input if the module doesn't use optional parameter
		$out .= $defaultInput;
	}
	return  $out;
}
#----------------------------------------------------------------------------
# css stylesheet
sub getCSS{
	my $out = "<style>
	BODY { background-color: buttonface; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	";
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= "
	TD, INPUT, SELECT { font-family: \"MS Sans Serif\"; font-size: xx-small; vertical-align: middle; }
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }";
	} else {	
		$out .= "
	TD, INPUT, P { font-family: sans-serif; font-size: small; vertical-align: middle; }	
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px; }";
	}
	$out .= "\n</style>";
	return $out;
}
# javascript
sub getJS{
	my $out = "";
	$out .= "\n<script langauge=\"JavaScript\" src=\"$config{server}{HTMLpath}/_ExSite/js/httprequest.js\"></script>";
	$out .= "\n<script language=\"JavaScript\">
<!--
	function insertdyncontent(){
		var name,param,url,requestUrl;
		name = document.forms[0].dyncontent.value;
		param = document.forms[0].param.value;
		requestUrl = '$config{server}{CGIpath}/$file?action=geturl&name=' + name;
		url = getRequestData(requestUrl);	
		window.opener.getDynContent(name,param,url);
		window.close();
	}
	function cancel(){
		window.returnValue = null;
		window.close();
	}
	function getDialogParam(divid,name){
		if (name == ''){return;}
		var requestUrl,thediv,inputParam;
		thediv = document.getElementById(divid);
		requestUrl = '$config{server}{CGIpath}/$file?action=getparam&name=' + name + '&section_id=$data{section}';
		inputParam = getRequestData(requestUrl);
		thediv.innerHTML = inputParam;
	}
//-->
</script>";
	return $out;
}
# get icon urls
sub getUrl{
	my ($name) = @_;
	my ($out,$test,$myobj);
	$test = eval("require Modules::$name");
	return "$config{server}{HTMLpath}/_ExSite/_Modules/icon.gif" if !$test;
	$myobj = eval("new Modules::$name");
	$out = "content-type: text/html\n\n";
	$out .= $myobj->icon;
	return $out;
}
# page header
sub PrintHeader {
	my (@links) = @_;
	my $out = "Content-type: text/html\n\n
<HTML>
<HEAD>
<TITLE>$title</TITLE>";
	$out .= &getCSS;
	$out .= &getJS;
	$out .= "
</head>
<body topmargin=\"0\" leftmargin=\"0\" style=\"border: 0; margin: 0;\" scroll=\"no\">\n";
	return $out;
}
# page footer
sub PrintFooter {
	return "\n</body>\n</html>";
}
sub PrintForm{
        my (@list,$ct);
	$ct = new ExSite::Content;
	@list = $ct->get_dcd($data{section});
	return "<p>Sorry, no web applications are available.</p>" if (scalar(@list) < 1);
        my $out = "<form onSubmit=\"insertdyncontent()\">\n";
        $out .= "\n<table border=\"0\">";
        $out .= "\n<tr><td colspan=\"2\"><b>$title</b><br><br></td></tr>";
	$out .= "\n<tr><td><b>Web Application: </b></td><td>";
	$out .= "<select name=\"dyncontent\" onChange=\"getDialogParam('param_input',this.value)\">";
	$out .= "<option value=\"\">== Select ==</option>";
	foreach my $name (@list){
		$out .= "<option value=\"$name\">$name</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>";
	$out .= "\n<tr><td width=\"100\" colspan=\"2\">";
	$out .= "<div id=\"param_input\">";
	$out .= "<table><tr><td><b>Parameter (optional):</b></td><td><input type=\"text\" name=\"param\" value=\"\"></td></table>";
	$out .= "</div>";
	$out .= "</td></tr>";	
        $out .= "\n<tr><td>";
        $out .= "<input type=\"button\" value=\"Insert\" onClick=\"insertdyncontent();\">";
        $out .= "</td><td>";
        $out .= "<input type=\"button\" value=\"Cancel\" onClick=\"cancel();\">";
        $out .= "</td></tr>";
        $out .= "\n</table>";
        $out .= "\n</form>\n";
	return $out;
}
### END
