#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_image.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use CGI::Carp qw(fatalsToBrowser);
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Content;
use ExSite::Section;
use ExSite::Page;
use ExSite::Image;
use myConfig;
use ExSite::Dialog;

use vars qw(%data $uid $access $OUT $title $db $file $selectedlib $defaultimagedim $dlg);

my $debug = 0;
print "content-type: text/html\n\n" if $debug;

&exsite_init;
$config{page_buffering} = 0;
%data = &DecodeInput;  # accepts input by post or query
$db = new ExSite::DB;
$access = $db->authorize;
$dlg = new ExSite::Dialog;
$title = "Insert Image/Document"; # title
$file = "dlg_image.cgi";
$defaultimagedim = 200;

if ($access==0){
	print "Location: login.cgi\n\n";
}elsif ($access < 2){
	$OUT .= &PrintHeader;
	$OUT .= "<p>Access denied.  Webmasters only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
}elsif ($data{action} eq "getimagelist"){
	# get image/document list
	print &getImageList;
}elsif ($data{action} eq "getimage"){
	# get single image/document
	print &getImage;
}elsif ($data{action} eq "geturl"){
	# get img url
	print &getUrl;
}else{
	#&findSectionID; # figure out section id]
	if (! $data{section_id}) {
	    $data{section_id} = $dlg->find_section_id(\%data);
	}
	$OUT .= &PrintHeader;
	# process image/document upload
	$OUT .= $data{action} eq "upload_file" ? &processUpload : "";
	$OUT .= &PrintForm;
	$OUT .= &PrintFooter;
	print $OUT;
}
#--------------------------------------------------------------------------------------
# figure out section id
sub findSectionID{
	my ($cdata);
	return if ($data{section_id});
	if (exists $data{data_id}) { 
	    $cdata = new ExSite::ContentData(id=>$data{data_id});
	}else {
	    $cdata = new ExSite::Page(id=>$data{cont_id});
	}
	$cdata->set_context();
	# set section id
	$data{section_id} = $cdata->get("section","section_id");
	return;
}
# css style sheet
sub getCss{
	my $out;
	$out .= "<style>
	BODY { background-color: buttonface; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }";
	# for msie
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= "	
	TD, INPUT, SELECT { font-family: \"MS Sans Serif\"; font-size: xx-small; 
		// vertical-align: middle; 
	}
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }";
	}else{	
		$out .= "
	TD, INPUT, P { font-family: sans-serif; font-size: small; 
		// vertical-align: middle; 
	}
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px;
	}";
	}
	$out .= "\n</style>";
	return $out;
}
# javascript
sub getJS{
	my $out;
	$out .= "\n<script langauge=\"JavaScript\" src=\"$config{server}{HTMLpath}/_ExSite/js/httprequest.js\"></script>";
	$out .= "\n<script language=\"JavaScript\">
<!--
	var isImage = 0;
	function insertimage(){
		var objName,objAlign,objId,objUrl,re,tempImage;
		re = new RegExp('checker');
		tempImage = new Image();
		objName = getContentName(document.forms[0].obj_id.value);
		objId = getContentID(document.forms[0].obj_id.value);
		objUrl = getRequestData('$config{server}{CGIpath}/$file?action=geturl&content_id=' + objId);
		objAlign = document.forms[0].align.value;
		tempImage.src = objUrl;
		if (re.test(objUrl)){
			tempImage.width = 100;
			tempImage.height = 50;
		}
		objWidth = tempImage.width;
		objHeight = tempImage.height;
		window.opener.getImage(isImage,objName,objUrl,objAlign,objWidth,objHeight);
		window.close();
	}
	function cancel(){
		window.returnValue = null;
		window.close();
	}
	function showimage(){
		var target,image,requestUrl,re;
		re = new RegExp('img');
		target = document.getElementById('disp');
		id = getContentID(document.forms[0].obj_id.value);
		if (!id){
			return;
		}
		requestUrl = '$config{server}{CGIpath}/$file?action=getimage&content_id=' + id;
		image = getRequestData(requestUrl);
		if (re.test(image)){
			showAlignmentForm();
		}else{
			hideAlignmentForm();
		}
		target.innerHTML = image;
	}
	function getImageList(){
		var target,lib_id,requestUrl,list;
		hideAlignmentForm();
		clearPreviewBlock();
		lib_id = document.forms[0].lib_id.value;
		target = document.getElementById('imageSelector');
		requestUrl = '$config{server}{CGIpath}/$file?action=getimagelist&page_id=' + lib_id;
		list = getRequestData(requestUrl);
		target.innerHTML = list;
	}
	function hideAlignmentForm(){
		var alignment = document.getElementById('img_alignment');
		alignment.style.visibility = 'hidden';
		isImage = 0;
	}
	function showAlignmentForm(){
		var alignment = document.getElementById('img_alignment');
		alignment.style.visibility = 'visible';
		isImage = 1;
	}
	function clearPreviewBlock(){
		var preview = document.getElementById('disp');
		preview.innerHTML = '';
	}
	function getContentID(val){
		var stringsplt = val.split(':');
		return stringsplt[0];
	}
	function getContentName(val){
		var result,stringsplt;
		result = '';
		stringsplt = val.split(':');
		for(var i = 1; i < stringsplt.length; i++){
			result += stringsplt[i];
		}
		return result;
	}
//-->
</script>\n";
	return $out;
}
sub PrintHeader {
	my (@links) = @_;
	my $out = "Content-type: text/html\n\n
<HTML>
<HEAD>
<TITLE>$title</TITLE>";
	$out .= &getCss;
	$out .= &getJS;
	$out .= "
</head>
<body topmargin=\"0\" leftmargin=\"0\" style=\"border: 0; margin: 0;\" scroll=\"no\">\n";
	return $out;
}
sub PrintFooter {
	return "\n</body>\n</html>";
}
# build insert image/document dialog
sub PrintForm{
	my $out = "\n<form onSubmit=\"insertimage()\">";
	$out .= "\n<table border=\"0\" cellspacing=\"10\" width=\"100%\">";
	$out .= "\n<tr><td colspan=\"2\">";
	$out .= "\n<h3 align=\"center\">$title</h3>";
	$out .= "\n</td></tr>";
	$out .= "\n<tr>";
	$out .= "<td valign=\"top\">";
	# library drop down list
	$out .= &getLibrarySelector;
	$out .= "\n<br><br>";
	# image drop down list
	$out .= &getImageSelector;
	$out .= "\n<br><br>";
	# image alignment drop down list
 	$out .= &getAlignmentRadio;	
	$out .= "\n<br><br>";
	$out .= "\n<input type=\"button\" value=\"Insert\" onClick=\"insertimage();\">";
	$out .= "\n&nbsp;&nbsp;&nbsp;&nbsp;";
        $out .= "\n<input type=\"button\" value=\"Cancel\" onClick=\"cancel();\">";        
	$out .= "\n</td>";
	$out .= "<td bgcolor=\"white\" valign=\"top\" width=\"150\" align=\"center\" style=\"border-left:solid;border-right:solid;border-top:solid;border-bottom:solid;\">";
	# preview area
	$out .= &getPreviewBlock;
	$out .= "\n</td>";
	$out .= "</tr>";	
	$out .= "\n</form>";
	# show upload form if it's turned on
	if ($config{form}{htmleditor}{op_image_upload}){
		$out .= "\n<tr><td colspan=\"2\">";
		$out .= "<hr>";
		$out .= "\n</td></tr>";
		$out .= "\n<tr><td colspan=\"2\">";	
		$out .= &uploadForm;
		$out .= "\n</td></tr>";
	}
	# set default library
	$out .= &setDefaultLib;
	return $out;
}
# document upload form
sub uploadForm{
	my $s = new ExSite::Section(id=>$data{section_id});
	my @libs = $s->fetch_libraries;
	my $out = "\n<form method=\"post\" enctype=\"multipart/form-data\" action=\"$config{server}{CGIpath}/$file\">";
	$out .= "\n<input type=\"hidden\" name=\"action\" value=\"upload_file\">";
	$out .= "\n<input type=\"hidden\" name=\"section_id\" value=\"$data{section_id}\">";
	$out .= "\n<input type=\"hidden\" name=\"cont_id\" value=\"$data{cont_id}\">";
	$out .= "\n<table border=\"0\" cellspacing=\"0\" width=\"100%\">";
	$out .= "\n<tr><td colspan=\"3\"><b>Add an image/document to a library:</b></td></tr>";
	$out .= "\n<tr><td>Library&nbsp;";
	$out .= "<select name=\"library\">";
	$out .= "<option value=\"\">== Select ==</option>";
	foreach my $l (@libs){
		if ($db->owns($l->{section_id})){
			$out .= "<option value=\"$l->{page_id}\">$l->{filename}</option>";
		}
	}
	$out .= "</select>";
	$out .= "</td>";
	$out .= "\n<td>";
	$out .= "File&nbsp;<input type=\"file\" name=\"fileupload\">";
	$out .= "</td>";
	$out .= "\n<td>";
	$out .= "<input type=\"submit\" value=\"Upload\">";
	$out .= "</td></tr>";
	$out .= "\n</table>";
	$out .= "\n</form>";
	return $out;
}
# javascript popup
sub alertPopup{
	my ($msg) = @_;
	my $out = "\n<script language=\"javascript\">\n<!--";
	$out .= "\nalert('$msg');";
	$out .= "\n// -->\n</script>";
	return $out;
}
# process form data
sub processUpload{
	my $space = "&nbsp;&nbsp;";
	return &alertPopup("Error: File upload failed.  No library is selected.") if !$data{library};
	return &alertPopup("Error: File upload failed.  No file is selected.") if !$data{fileupload};
	my ($rawdata,$size,$mimedata,$file,$continue,$content,$filename,@path,$img,$fdata);
	$rawdata = undef;
	$size = 0;
	$mimedata = "";
	$file = $data{fileupload};
	$continue = 1;
	# read upload data
	while ($continue){
		$continue = read($file,$rawdata,1024,$size);
		$size += $continue;
	}
	return &alertPopup("Error: File upload failed.  No file data was received.") if !$size;
	@path = split /[:\\\/]/, $file;        
        if ($path[-1]){
		$filename = &url_escape($path[-1]);
	}
	$img = new ExSite::Image($filename,$rawdata);
	$fdata = $img->encode;
	$content = new ExSite::Content;
	# insert form data
	$content->make_content({name=>$filename,page_id=>$data{library}},{fdata=>$fdata});
	return &alertPopup("$filename has been uploaded successfully.");
}
# create an image/document list
sub getImageList{
	my ($out,$list);
	$list = $db->fetch_match("content",{page_id=>$data{page_id}});
	$out = "content-type: text/html\n\n";
	$out .= "\n<select name=\"obj_id\" onChange=\"showimage()\">";
	$out .= "\n<option value=\"\">== Select ==</option>";
	foreach my $l (@$list){$out .= "\n<option value=\"$l->{content_id}:$l->{name}\">$l->{name}</option>";}
	$out .= "\n</select>";
	return $out;
}
sub getImage{
	#return &getObject("image",$data{content_id});
	return $dlg->get_object("image",$data{content_id});
}
sub getUrl{
	#return &getObject("url",$data{content_id});
	return $dlg->get_object("url",$data{content_id});
}
# create a library drop down list
sub getLibrarySelector{
	my ($out,@libs,$s);
	@libs = @{$dlg->fetch_library($data{section_id})};
	$out .= "\n<b>Use library:</b><br>";
	$out .= "\n<select id=\"lib_id\" name=\"lib_id\" onChange=\"getImageList()\">";
	$out .= "\n<option value=\"\">== Select ==</option>";
	foreach my $l (@libs){
		$out .= "\n<option value=\"$l->{page_id}\"";
		# set first image library as the default
		if ($l->{filename} =~ /image/i && !$selectedlib){
			$out .= " selected";
			$selectedlib = $l->{page_id};
		}
		$out .= ">$l->{filename}</option>";
	}
	$out .= "\n</select>";
	return $out;
}
# create a default image/document drop down list
sub getImageSelector{
	my ($dlist) = @_;
	my $out = "";
        $out .= "\n<b>Select an image/document:</b><br>";
	$out .= "\n<div id=\"imageSelector\">";
       	$out .= "\n<select name=\"obj_id\" onChange=\"showimage()\">";
       	$out .= "\n<option value=\"\">== Select ==</option>";        
       	$out .= "\n</select>";
	$out .= "\n</div>";
	return $out;
}
# create an image alignment drop down list
sub getAlignmentRadio{
	my $out = "";
	$out .= "\n<div id=\"img_alignment\" style=\"visibility: hidden;\">";
	$out .= "\n<b>Image Alignment:</b><br>";
	$out .= "\n<select name=\"align\">";
	$out .= "\n<option value=\"\">== Select ==</option>";
	$out .= "\n<option value=\"left\">Left</option>";
	$out .= "\n<option value=\"right\">Right</option>";
	$out .= "\n</select>";
	$out .= "\n</div>";
	return $out;
}
# preview area
sub getPreviewBlock{
	my $out = "";
	$out .= "\n<b><center>Preview</center></b><br>";
        $out .= "<div id=\"disp\"></div>";
	return $out;
}
# fetch image/document list for the default library
sub setDefaultLib{
	my $out = "";
	if ($selectedlib){
		$out .= "\n<script language=\"javascript\">";
		$out .= "\n<!--";
		$out .= "\ndocument.forms[0].lib_id.value = $selectedlib;";
		$out .= "\ngetImageList();";
		$out .= "\n//-->";
		$out .= "\n</script>";
	}
	return $out;
}
### END
