#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_link.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Dialog;
use myConfig; 

use vars qw(%data $uid $access $OUT $operation $db $js $htop $hbottom $file $dlg);

&exsite_init;
$config{page_buffering} = 0;
$file = "dlg_link.cgi";
%data = &DecodeInput;  # accepts input by post or query
$db = new ExSite::DB;
$access = $db->authorize;
$dlg = new ExSite::Dialog;
$operation = ($data{url})? "Edit" : "Insert";

if ($access==0) {
	print "Location: login.cgi\n\n";
} elsif ($access<2) {
	&PrintHeader;
	$OUT .= $htop.$hbottom;
	$OUT .= "<p>Access denied.  Webmasters only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
} elsif ($data{action} eq "getimagelist"){
	print &getImageList();
} elsif ($data{action} eq "checkfile"){
	print &checkValidFile($data{section},$data{name});
} else {
	&PrintHeader;
	my @links = &GetSiteLinks;
	$OUT .= $htop.$js.$hbottom;
	$OUT .= &PrintForm(@links);
	$OUT .= &PrintFooter;
	print $OUT;
}

#-----------------------------------------------------------------------------------
sub checkValidFile{
	my ($sectionid,$filename) = @_;
	my $out = "content-type: text/html\n\n";
	return $out.$dlg->is_exist($sectionid,$filename);
}
sub GetSiteLinks {
	my (@links,$page_list,$page,$sql,$jscript);
	$jscript = "var pageref = new Array();\n";
	# fetch pages
	$page_list = $dlg->fetch_page(\%data);
	foreach my $p (@$page_list) {
		$jscript  .= "pageref[\"$p->{filename}\"] = $p->{page_id};\n";
		my $url = "$p->{filename}";
		my $title = "$p->{title}";
		# the rest go in order of the fetch, which is currently alphabetical
		push(@links,{url=>$url, text=>$title});
	}
	$js = $jscript.$js;
	return @links;
}

sub PrintHeader {
	$htop = "Content-type: text/html\n\n
<HTML>
<HEAD>
<TITLE>$operation Link</TITLE>";
if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
$htop .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: \"MS Sans Serif\"; font-size: xx-small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
</style>";
}else{
$htop .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: sans-serif; font-size: small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px;
	}
</style>";
}
$htop .= "<script language=\"JavaScript\">
<!--\n";
$js .= "\tfunction attr(name, value) {
		if (!value || value == \"\") return \"\";
		return ' ' + name + '=\"' + value + '\"';
	}
	function insertLink() {
		var linkname = document.forms[0].linkURL.value;
		var newWin = document.forms[0].newwindow.checked || 0;
		var isImgDocObject = getRequestData('$config{server}{CGIpath}/$file?action=checkfile&section=$data{section_id}&name=' + linkname);
		window.opener.getLink(linkname,document.forms[0].linkText.value,newWin,isImgDocObject);
		window.close();
	}
	function cancel() {
		window.returnValue = null;
		window.close();
	}
	function selectMade() {
		var selector = document.forms[0].linkSelect;  
		var choice = selector.options[selector.selectedIndex];
		if (choice.value) {
			document.forms[0].linkURL.value = choice.value;
		}
	}
	function selectLibrary(){
		var selector = document.forms[0].obj_id;
		var choice = selector.options[selector.selectedIndex];
		if (choice.value){
			document.forms[0].linkURL.value = choice.value;
		}
	}
	function urlsubmit() {
		var key = document.forms[0].linkURL.value;
		if (key){
			if (pageref[key]){
				document.forms[0].linkURL.value = pageref[key];
			}
		}
	}
	function insertsubmit(){
		urlsubmit();
		insertLink();
	}
";
$hbottom .= "\n//-->
</script>
</head>
	<body topmargin=\"0\" leftmargin=\"0\" style=\"border: 0; margin: 0;\" scroll=\"no\">\n";
}

# create a library drop down list
sub getLibrarySelector{
	my ($out,@libs,$s);
	@libs = @{$dlg->fetch_library($data{section_id})};
	$out .= "\n<select id=\"lib_id\" name=\"lib_id\" onChange=\"getImageList()\">";
	$out .= "\n<option value=\"\">== Select Library ==</option>";
	foreach my $l (@libs){
		my $name = $dlg->truncate_text($l->{filename});
		$out .= "\n<option value=\"$l->{page_id}\"";
		# set first image library as the default
		$out .= ">$name</option>";
	}
	$out .= "\n</select>";
	return $out;
}

sub getImageList{
	my ($out,$list);
	$list = $db->fetch_match("content",{page_id=>$data{page_id}});
	$out = "content-type: text/html\n\n";
	$out .= "\n<select name=\"obj_id\" onChange=\"selectLibrary()\">";
	$out .= "\n<option value=\"\">== Select Image/Document ==</option>";
	foreach my $l (@$list){
		my $name = $dlg->truncate_text($l->{name});
		$out .= "\n<option value=\"$l->{name}\">$name</option>";
	}
	$out .= "\n</select>";
	return $out;
}

# javascript
sub getJS{
	my $out;
	$out .= "\n<script langauge=\"JavaScript\" src=\"$config{server}{HTMLpath}/_ExSite/js/httprequest.js\"></script>";
	$out .= "\n<script language=\"JavaScript\">
<!--
	function getImageList(){
		var target,lib_id,requestUrl,list;
		lib_id = document.forms[0].lib_id.value;
		target = document.getElementById('imageSelector');
		requestUrl = '$config{server}{CGIpath}/$file?action=getimagelist&page_id=' + lib_id;
		list = getRequestData(requestUrl);
		target.innerHTML = list;
	}
//-->
</script>\n";
	return $out;
}

sub PrintFooter {
	return "\n</body></html>";
}

sub PrintForm {
	my (@links) = @_;
	my $spaces = "<img width=\"10\" height=\"10\" border=\"0\" src=\"$config{server}{HTMLpath}/_ExSite$config{server}{imgdir}/null.gif\">";
	my $out = &getJS();
	$out .= "
<form>
<table border=\"0\" class=\"dlg\" cellpadding=\"0\" cellspacing=\"2\" width=\"100%\" height=\"100%\">
<tr>
 <td colspan=\"5\">
  <table border=\"0\" width=\"100%\"><tr><td>$operation&nbsp;Link</td><td valign=\"middle\" width=\"90%\"><hr width=\"100%\"></td></tr></table>
 </td>
</tr>
<tr>
 <td width=\"10\">&nbsp;</td>
 <td width=\"150\">Link Text:</td>
 <td colspan=\"3\" valign=\"middle\"><input type=\"text\" name=\"linkText\" value=\"$data{title}\" size=\"50\"></td>
</tr>
<tr>
 <td colspan=\"5\">
  <table border=\"0\" width=\"100%\"><tr><td>Link To</td><td valign=\"middle\" width=\"90%\"><hr width=\"100%\"></td></tr></table>
 </td>
</tr>
<tr>
 <td width=\"10\">&nbsp;</td>
 <td width=\"150\">Internal Page:</td>
 <td colspan=\"3\" valign=\"middle\">
 <SELECT NAME=\"linkSelect\" onChange=\"selectMade()\">";
	if (scalar @links) {
		$out .= "<OPTION value=\"\">==Select Internal Page ==</OPTION>\n";
	} else {
		$out .= "<OPTION value=\"\">==No Internal Pages Found==</OPTION>\n";
	}
	foreach my $link (@links) {
		my $linktext = $dlg->truncate_text($link->{text});
		$out .= "<OPTION value=\"$link->{url}\">$linktext</OPTION>\n";
	}
	$out .= "
 </SELECT>
 </td>
</tr>
<tr>
 <td width=\"10\">&nbsp;</td>
 <td width=\"150\">or Image/Document:</td>
 <td width=\"250\">";
	$out .= &getLibrarySelector();
 	$out .= "</td>
 <td colspan=\"2\"><div id=\"imageSelector\">
<select name=\"obj_id\" onChange=\"showimage()\">
<option value=\"\">== Select Image/Document ==</option>
</select>
</div></td>
</tr>
<tr>
\n";
	$out .= "<tr>
 <td width=\"10\">&nbsp;</td>
 <td width=\"150\">or URL</td>
 <td colspan=\"3\" valign=\"middle\"><input type=\"text\" name=\"linkURL\" value=\"$data{url}\" size=\"50\"></td>
</tr>
 <td width=\"10\">&nbsp;</td>
 <td colspan=\"4\"><input type=\"checkbox\" name=\"newwindow\"> Open this link in a new window</td>
</tr>
<tr>
 <td>&nbsp;</td>
 <td colspan=\"4\"><br>
  <input class=\"button\" type=\"button\" value=\"Submit\" title=\"Submit\" onclick=\"insertsubmit()\">
  <input class=\"button\" type=\"button\" value=\"Cancel\" title=\"Cancel Dialog\" onclick=\"cancel()\">
 </td>
</tr>
</table>
</form>";
	return $out;
}
### END
