#!/usr/bin/perl
#-----------------------------------------------------------------------
# echo.cgi - copy content to browser (utility used by WYSIWYG html editor)
#
# usage:
#
# echo.cgi?tab={TABLE}&col={COLUMN}&id={RECORD_ID}  # echo database field
# echo.cgi/path/to/file  # echo file in htdocs area
# echo.cgi?url={URL}  # echo remote URL
#
# extra arguments:
#  cms = if true, perform limited CMS substitutions on content
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;
use ExSite::Misc;
use ExSite::Content;
use vars qw(%input $db);

&exsite_init;

%input = &DecodeQuery;
$db = new ExSite::Form;

my $access = $db->authorize;

print "Content-type: text/html\n\n";
print "<html>\n<head>\n\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">\n</head>";	
if ($access >= 2 && ($input{url} ||
		     ($input{tab} && $input{col} && $input{id}) ||
		     $ENV{PATH_INFO})) { 
    print &get_html; 
}
elsif ($ENV{REQUEST_URI} =~ /$config{echo}{public_url}/) { 
    print &get_html; 
}
print "\n</html>\n";

# get contents

sub get_html {
    my $content;

    # case 1 : a file on the local machine

    if ($ENV{PATH_INFO}) {
	my $file = "$config{server}{HTMLroot}$config{server}{HTMLpath}$ENV{PATH_INFO}";
	#	 read file contents
	open F, "<$file";
	my $continue = 1;
	my $size = 0;
	while ($continue) {
	    $continue = read(F,$content,1024,$size);
	    $size += $continue;
	}
	close F;
    }

    # case 2 : a remote URL

    elsif ($input{url}) {
	require LWP::UserAgent;
	# fetch the URL
	my $ua = new LWP::UserAgent;
	$ua->timeout(10);
	$ua->env_proxy;
	my $response = $ua->get($input{url});
	if ($response->is_success) {
	    print $response->content;
	}
	else {
	    print $response->status_line;
	}
    }

    # case 3 : a field in the database

    else {
	if (! $input{tab} || ! $input{col} || ! $input{id}) {
	    # return "Error - missing arguments"; 
	    # WARNING: no data destination given;  action script must 
	    # know how to process the data.
	    return "";
	} 
	if (! exists $db->{map}->{tables}->{$input{tab}} ) {
	    return "Error - no such table as <b>$input{tab}</b>";
	} 
	#if ($db->{map}->get($table,$column,"datatype") ne "html" ) {
	#	return "Error - column <b>$column</b> in table <b>$table</b> is not of type <b>html</b>";
	#} 
	if ($input{cms} && $input{tab} eq "content_data") {
	    # potential for CMS substitutions
	    require ExSite::ContentData;
	    my $cdata = new ExSite::ContentData(id=>$input{id});
	    if ($cdata->content_type() eq "text/html") {
		# expand the content so it looks more realistic in the editor
		# - use dummy placeholders for content, modules so that we
		#   know how to convert them back to tags
		# - use real URL replacements, which look better, but use
		#   dynamic mode so we know how to convert them back to tags
		$content = $cdata->expand(
				 method=>"url,dummy-content,dummy-module",
				 mode=>"dynamic");
	    }
	}
	else {
	    if ($input{id} =~ /^\d+$/) {
		my %data = $db->fetch($input{tab},$input{id});
		if (! exists $data{$input{col}}) {
		    return "Error - cannot retrieve column <b>$input{col}</b> in table <b>$input{tab}</b> where <b>id=$input{id}</b><br>You do not have permission to access this data.";
		} 
		$content = $data{$input{col}};
	    }
	}
    }
    return $content;
}
