#!/usr/bin/perl
#-----------------------------------------------------------------------
# getdata.cgi : simple datum fetching script
#
# this script fetches a single datum and displays it with an
# appropriate content-type.  It takes a query string of the form:
#     tab=<table>&id=<record>&col=<data_column>[&type=mime-type-id]
# "col" is the column name containing the data to return.
# "type" can be a mime type (/.+\/.*/);  if not given, it will be
# guessed from the data.  text/plain is the default for non-file datatypes.
#
# Typically the script will be used to fetch images from a database
# for inlining into a web page, using a method something like this:
#
# <IMG SRC=/cgi/getdata.cgi?tab=image&id=999&col=imgdata&type=mime>
#
# It can also be used to fetch non-HTML documents for download:
#
# <A HREF=/cgi/getdata.cgi/filename?tab=document&id=999&col=document>...</A>
#
# (here, "filename" is an arbitrary string that browsers will use as the
# default filename for the saved file.)
#
# It could even be used to fetch whole web pages, eg.
#
# Location: /cgi/getdata.cgi?tab=webpage&id=999&col=html&type=text/html
#
# Or even just to serve as a HTTP "peek" function, eg.
#
# http://www.myhost.com/cgi/getdata.cgi?tab=abc&id=999&col=column
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use MIME::Base64;
use ExSite::Config;# qw(:OLD);
use ExSite::Misc;
use ExSite::DB;

&exsite_init;
#&set_old_config_vars;
my %q = &DecodeQuery;
my (%data, $outdata);
my $db = new ExSite::DB;

# check permissions:
my ($approved,$readcol_ok);

# ignore unreferred calls
if (($config{auth}{referer_check} && $ENV{HTTP_REFERER}) || 
    !$config{auth}{referer_check}) {
    # 1. permission to read this record
    $approved = $db->approve("read",$q{tab},$q{id});
    # 2. permission to read this column
    $readcol_ok = $db->{map}->get($q{tab},$q{col},"read") <= $db->authorize;
}

if ($approved && $readcol_ok) {
    # fetch the requested record
    if ($q{tab} && $q{id} && $q{col}) {
	%data = $db->fetch($q{tab},$q{id});
    }
    # determine the mime type
    if ($q{type} =~ /.+\/.+/) {
	# type has been passed in the QUERY_STRING
	print "Content-type: $q{type}\n\n";
	$outdata = $data{$q{col}};
    }
    elsif ($db->{map}->get_column_attr($q{tab},$q{col},"datatype") =~ /file$/) {
	# type should be guessable from the filename
	my ($filename, $content) = split /$config{form}{sepchar}/, $data{$q{col}}, 2;
	my $mimetype = &MimeType($filename);
	print "Content-type: $mimetype\n\n";
	$outdata = decode_base64($content);
    }
    else {
	# unknown type; display the raw datum as text/plain
	print "Content-type: text/plain\n\n";
	$outdata = $data{$q{col}};
    }
}
else {
    print "Content-type: text/plain\n\n";
    $outdata = "Permission denied.\n";
}

# display the data
if ($outdata) {
    print $outdata;
}
