#!/usr/bin/perl
#-----------------------------------------------------------------------
# home.cgi - ExSite WebTop
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::PageBuf;
use ExSite::Misc;
use ExSite::Report;

use vars qw($db %query $popup_flag);

&exsite_init;
# connect to database
$db = new ExSite::DB;

# enable page buffering for better redirect control
$config{page_buffering} = 1;
my $p = new ExSite::PageBuf(title=>$config{site}{name}, header=>0);

my $access = $db->authorize;
my $webtop_access = &webtop_access;
my $webtop = 0;

if ($webtop_access == 1) {

    # Web Top

    $webtop = 1;
    $p->{headtype} = $webtop;
    $p->header;
    $p->print("<table cellspacing=0 cellpadding=20 border=0><tr>\n");

    opendir MODDIR, "$config{server}{CGIroot}/Modules";

    my ($mod,$napp);
    while ($mod = readdir MODDIR) {
	next if ($mod !~ /\.pm$/);    # only look at Perl modules
	# extract module name
	my $modname = $mod;
	$modname =~ s/\.pm$//;
	my $stat = eval "require Modules::$modname";
	if ($stat) {
	    # instantiate the module
	    my $mod = eval "new Modules::$modname";
	    # look for a control panel
	    if ($mod) {
		my $panel = $mod->ioctl('ControlPanel');
		if ($panel) {
		    # a control panel exists for this module;
		    
		    # check for authorization
		    my $allow = (! $mod->ioctl('isRestricted')) || 
			$db->license_control_panel($modname);

		    if ($allow) {
			# find a suitable icon
			my $icon = "$config{server}{HTMLpath}/_Modules/$modname/icon.gif";
			if (! -e "$config{server}{HTMLroot}$icon") {
			    $icon =~ s/\.gif$/\.png/;
			    if (! -e "$config{server}{HTMLroot}$icon") {
				# no icon; use default ExSite app icon
				$icon = "$config{server}{HTMLpath}/_Modules/icon.gif";
			    }
			}
		    
			# find the control panel URL
			my $url;
			if (ref $panel eq "CODE") {
			    $url = "$config{server}{CGIpath}/ctrl-panel.cgi/$modname";
			}
			else {
			    # assume ioctl has returned the control panel url
			    $url = $panel;
			}
			$p->print("<td align=center><a class=webtop href=$url><img src=$icon height=64 width=64 border=0><br>");
			# get application name
			my $app = $mod->ioctl('ModuleName');
			if (!$app) { $app = $modname; }
			$p->print("$app</a></td>\n");
			$napp++;
			# icon grid width is $config{webtop}{x} 
			if ($napp > 0 & $napp % $config{webtop}{x} == 0) {
			    $p->print("</tr><tr>\n");
			}
		    }
		}
	    }
	}
    }
}
elsif ($webtop_access == -1) {

    # the user has less than administrator access...

    # no webtop :-(

    $p->{headtype} = $webtop;
    $p->{title} = "Select Web Site";
    $p->header;
    $p->print("<table cellspacing=0 cellpadding=20 border=0><tr>\n");

    require ExSite::Section;

    # 1) find out which websites they are connected to

    my %user = $db->my_user_record;
    my @sites = $db->my_real_sites;

    # TODO: if there is only one site, jump to it directly

    if (@sites == 0) {
	$p->print("<td>\n<h3>No Access</h3>\n<p>Sorry, but you have not been granted management priveleges to any websites.  Please check with your system administrator.</p>\n<p><a href=\"$config{server}{CGIpath}/logout.cgi\" style=\"border-width:1px; border-color:black; border-style:solid;\">&nbsp;&nbsp;Okay&nbsp;&nbsp;</a><dl>\n");
    }
    else {
	$p->print("<td>\n<h3>$user{name}'s Websites</h3>\n<p>You are associated with the following websites. Please select which one you want to open.</p>\n<dl>\n");
	foreach my $site (@sites) {
	    my $s = new ExSite::Section(section=>$site);
	    $s->set_context;
	    my $section = $s->get;
	    my $page_id = $s->id("page");
	    my $url;
	    if ($access == 2) {
		if ($page_id) {
		    $url = "$config{server}{CGIpath}/page.cgi?_id=$page_id&_cmsmode=1";
		    if (@sites = 1) {
			$p->location($url);
		    }
		}
		else {
		    $url = "$config{server}{CGIpath}/ctrl-panel.cgi/CMS?path=/$s->{data}{section_id}";
		    if (@sites = 1) {
			$p->location($url);
		    }
		}
	    }
	    else {
		# no CMS interface for level 1 users
		$url = $s->httppath;
	    }
	
	    $p->print("<dt><a href=$url>$section->{title}</a></dt>\n<dd>$section->{description}</dd>\n");
	}
    }

    $p->print("</dl>\n</td>\n");
}
else {
    $p->{headtype} = $webtop;
    $p->{title} = "Permission Denied";
    $p->header;
    $p->print("<table cellspacing=0 cellpadding=20 border=0><tr>\n");
    $p->print("<td><h3>You do not have permission for this operation.</h3></td>\n");
}

$p->print("</tr></table>\n");

$p->show;

# webtop_access
# returns -1 for denial to webtop only
#          0 for complete denial of access
#          1 for access to webtop
# customizable through handler

sub webtop_access {
    # use a handler in cases where simple access-level control is insufficient
    my $stat = $db->run_handler("webtop_access");
    return $stat if (defined $stat);
    # default to access level control
    my $access = $db->authorize;
    my $min_access = $config{webtop_access} || 3;
    if ($access) {
	return ($access >= $min_access) ? 1 : -1;
    }
    return 0;
}

### END

