#!/usr/bin/perl
#-----------------------------------------------------------------------
# login.cgi : authenticate a user and set their identity cookie
#
# This script accepts a "login" and "password" from an HTML form,
# and authenticates them against values looked up in a database.
# If a match is found, an identity cookie is set for future
# authentication.
#
# If called with no input, the script will throw up a simple login
# form. The site developer may code similar login forms into any HTML
# pages they choose.
#
# The script accepts input by POST or via the QUERY_STRING.  That is,
# even in the absence of a login form, login can still be done using
# an URL like /cgi/login.cgi?login=my_name&password=my_pwd
# (Obviously, you won't want to hard-code sensitive passwords into 
# publicly visible HTML.)
#
# Following a successful authentication, the user is redirected to
# a login page.  Default login targets are defined in 
# @ExSite::Config::login_destination, but these may be overridden by 
# supplying a "goto" argument in the login.cgi input data.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;

use vars qw(%data $uid $access);

&exsite_init;
%data = &DecodeInput;  # accepts input by post or query

if (exists $data{login}) {
    my $db = new ExSite::DB;
    if (my $access = $db->login($data{login},$data{password})) {
	my $destination = 
	    $data{goto} || 
	    $config{auth}{login_destination}[$access];
	print "Location: $destination\n\n";
    }
    else {
	&DenyLogin($db->show_diagnostics("warn","HTML"));
    }
}
else {
    &PromptLogin;
}

sub PromptLogin {
    my $db = new ExSite::DB;
    my $access = $db->authorize();
    if ($access) {
	# user already logged in
	my $destination = $data{goto} || $config{auth}->{login_destination}->[$access];
	print "Location: $destination\n\n";
    }
    else {
	print <<EOF;
Content-type: text/html

<HTML>
<HEAD>
<LINK REL="stylesheet" TYPE="text/css" HREF="$config{server}{HTMLpath}/_ExSite/css/ExSite.css">
<TITLE>$config{site}->{name} Login</TITLE>
</HEAD>
<BODY>
<CENTER>
<P>$config{site}->{logo}</P>
<h2>$config{site}->{name} Login</h2>
<FORM METHOD=POST ACTION=$config{server}->{CGIpath}/login.cgi>
<TABLE CLASS="form">
<TR><TD>Login:</TD><TD><INPUT TYPE=text NAME=login></TD></TR>
<TR><TD>Password:</TD><TD><INPUT TYPE=password NAME=password></TD></TR>
</TABLE>
<P><INPUT TYPE=submit VALUE=" Log in "></P>
</form>
</CENTER>
</BODY>
</HTML>
EOF
    }
}

sub DenyLogin {
  &page_header("Login Denied");
  print "<H3>Login Denied</H3>\n";
  print @_;
  print "<P><A HREF=\"javascript:history.back()\">Try again</A></P>\n";
  &page_footer;
}

### END

