#!/usr/bin/perl
#-----------------------------------------------------------------------
# page.cgi - generate and display a web page
#
# USAGE:
#    page.cgi?_id=xxx - posts page xxx to stdout
#    page.cgi/path/to/file.html - ditto, but uses published version if it can
#
# Parameters:
#    _id = id number of page to display
#    _cms = inlined editing flag (enable editing if true)
#    _cmsmode = persistent inlined editing flag (enable editing for all pages)
#
# Other parameters are passed through to dynamic content objects that
# are embedded in the page.  By convention, parameters directed at page.cgi
# have a preceding underscore;  other parameters are intended for embedded
# dynamic content modules.
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use ExSite::Config;
use ExSite::Form;
use ExSite::Page;
use ExSite::Session;
use ExSite::Util;
use ExSite::Input;

&exsite_init;
my $db = new ExSite::Form;
my $in = new ExSite::Input;
my %query = %{$in->query};
my $html;
# create a null page
my $p = new ExSite::Page;

# check to see if we need embedded CMS features in this page;

my $cmsflag = 0;
my $access = $db->authorize;
if ($access >= 2) {
    if (exists $query{_cms}) {
	# _cms parameter can override all settings for this page only
	$cmsflag = $query{_cms};
    }
    elsif (exists $query{_cmsmode}) {
	# _cmsmode parameter can set mode for all pages
	$cmsflag = $query{_cmsmode};
	# turn inlined CMS features on/off for subsequent pages
	my $cookieval = &get_cookie("ExSiteCMS");
	if ($cmsflag != $cookieval) {
	    # change the cookie value
	    &edit_cookie("ExSiteCMS", $query{_cmsmode});  # this page
	    $p->cookie("ExSiteCMS", $query{_cmsmode});    # subsequent pages
	}
    }
    else {
	# otherwise check if the mode was set previously
	$cmsflag = &get_cookie("ExSiteCMS");
    }
}

# find the page and set it up

my $makepage = 0;
if ($query{_id}) {
    $p->setup(id=>$query{_id},save=>1);
    $makepage = 1;
}
elsif ($ENV{PATH_INFO}) {
    my $path = $ENV{PATH_INFO};
    if (-e "$config{server}->{HTMLroot}$path") {
	$p->redirect($path);
    }
    else {
	my @path = split(/\//,$path);
	shift @path;
	my $section_id;
	if (@path > 1) {
	    my $section_name = shift @path;
	    my @section = $db->fetch_match("section",{name=>$section_name});
	    $section_id = $section[0]->{section_id};
	}
	my @page = $db->fetch_match("page",{filename=>$path[-1],section_id=>$section_id});
	my $pid = $page[0]->{page_id};
	$p->setup(id=>$query{_id},save=>1);
	$makepage = 1;
    }
}
else {
    $p->errorpage("<H3>Error</H3><P>No page to display!</P>\n");
}

# generate the page

if ($makepage) {
    # save page pointer for benefit of DCDs
    $share{Page} = $p;
    $p->set_context;

    if ($p->authorize) {
	# insert all the content
	$p->expand(cms=>$cmsflag);
    }
    else {
	# permission denied!  Require user to log in before proceeding.
	my $goto = $config{server}{server}.$ENV{REQUEST_URI};
	$p->errorpage("<CENTER>\n<h2>Login</h2>\n".
		      $share{DB}->login_form("",$goto).
		      "</CENTER>\n");
    }
}

$p->show;

&exsite_close;
